function [jn0,jnhn]=Besseljnhn(nMax,K,freq,radius)
%for use with ECAH ultrasound scattering calculations
%calculate the Bessel functions jn and hn and derivatives up to the max order
%also the jn and its derivative for the incoming wave
%
%inputs are the nMax: the maximum order needed,
%K(iSubs,iMode,iFreq): the wavenumbers for all substances, modes, and frequencies
%freq: the frequency values
%radius: the particle sizes (in metres)
%
%outputs are jnhn(iSubs,iMode,iDeriv,n,iFreq,iRad) the relevant Bessel
%functions and derivative for each mode and substance
%and its derivative at the particle surface
%for the incident field it is jn0(iDeriv,n,iFreq,iRad)
%by Valerie Pinfield, 2004
%
%for Bessel function for the incoming wave, the derivative relates to 
% the previous function i.e.jnp=j(n-1)-(n+1)*jn/x
%the zero order one needs j(-1)
%Need hn and derivatives for substance 1 (continuous phase), all modes
%jn and derivatives for all dispersed phases and all modes
%and jn and derivatives for the incident wave (prop mode 1)


numFreq=size(freq,2);
numRad=size(radius,2);
numSubs=size(K,1); %number of substances
spherFactor=sqrt(pi/2); % to convert Jn to spherical bessel function jn

for iFreq=1:numFreq
    for iRad=1:numRad
        %calculate the continuous phase first, Hankel, hn functions 
        %(not incident wave)
        Ka=K(1,:,iFreq)*radius(iRad); 
        rootKa=sqrt(Ka);
        %zero order function, n=0 stored in n+1 array element
        jnhn(1,:,1,1,iFreq,iRad)=-i*exp(i*Ka)./Ka; %h0
        jn0(1,1,iFreq,iRad)=sin(Ka(1))/Ka(1); %incident field j0
        
        for n=1:nMax    %do each order : calc derivative on next loop
            jnhn(1,:,1,n+1,iFreq,iRad)=besselh(n+0.5,1,Ka)*...
                spherFactor./rootKa; %hn for subs 1
            %calculate derivatives for previous order
            jnhn(1,:,2,n,iFreq,iRad)=(n-1)*... %1st derivative hn'
                (jnhn(1,:,1,n,iFreq,iRad)./Ka)-...
                jnhn(1,:,1,n+1,iFreq,iRad);
            jnhn(1,:,3,n,iFreq,iRad)=(n-1)*(n-2)*...%2nd derivative hn''
                jnhn(1,:,1,n,iFreq,iRad)./(Ka.*Ka)-...
                jnhn(1,:,1,n,iFreq,iRad)+2*jnhn(1,:,1,n+1,iFreq,iRad)./Ka; 
 %incident field, all jn Bessel functions with argument Ka(1) i.e. mode 1
            jn0(1,n+1,iFreq,iRad)=besselj(n+0.5,Ka(1))...
                    *spherFactor/rootKa(1); 
                %calculate derivative for previous order
            jn0(2,n,iFreq,iRad)=(n-1)*(jn0(1,n,iFreq,iRad)/Ka(1))-...
                    jn0(1,n+1,iFreq,iRad); %first derivative
            jn0(3,n,iFreq,iRad)=(n-1)*(n-2)*...%2nd derivative
                    jn0(1,n,iFreq,iRad)/(Ka(1)*Ka(1))-...
                    jn0(1,n,iFreq,iRad)+...
                    2*jn0(1,n+1,iFreq,iRad)/Ka(1); 

        end
        %now calc derivative for the highest order (stored in index nMax+1)
        jnhn(1,:,2,nMax+1,iFreq,iRad)=(nMax)*...
            jnhn(1,:,1,nMax+1,iFreq,iRad)./Ka-...
            besselh(nMax+1.5,1,Ka)*spherFactor./rootKa;
        jnhn(1,:,3,nMax+1,iFreq,iRad)=nMax*(nMax-1)*...%2nd derivative
                jnhn(1,:,1,nMax+1,iFreq,iRad)./(Ka.*Ka)-...
                jnhn(1,:,1,nMax+1,iFreq,iRad)+...
                2*besselh(nMax+1.5,1,Ka)*spherFactor./(rootKa.*Ka);
       %incident field
        jn0(2,nMax+1,iFreq,iRad)=(nMax)*...
                jn0(1,nMax+1,iFreq,iRad)/Ka(1)...
                -besselj(nMax+1.5,Ka(1))*spherFactor/rootKa(1);
        jn0(3,nMax+1,iFreq,iRad)=nMax*(nMax-1)*...%2nd derivative
                jn0(1,nMax+1,iFreq,iRad)/(Ka(1)*Ka(1))-...
                jn0(1,nMax+1,iFreq,iRad)+...
                2*besselj(nMax+1.5,Ka(1))*spherFactor/(rootKa(1)*Ka(1));
        
        for iSubs=2:numSubs %dispersed phases
            Kpa=K(iSubs,:,iFreq)*radius(iRad);  
            rootKpa=sqrt(Kpa);    

            %zero order function, n=0 stored in n+1 array element
            jnhn(iSubs,:,1,1,iFreq,iRad)=sin(Kpa)./Kpa; %j0
            for n=1:nMax    %do each order : calc derivative on next loop
                jnhn(iSubs,:,1,n+1,iFreq,iRad)=besselj(n+0.5,Kpa)...
                    *spherFactor./rootKpa; %jn for substances2+
                %calculate derivative for previous order
                jnhn(iSubs,:,2,n,iFreq,iRad)=(n-1)*...
                    (jnhn(iSubs,:,1,n,iFreq,iRad)./Kpa)-...
                    jnhn(iSubs,:,1,n+1,iFreq,iRad);
                jnhn(iSubs,:,3,n,iFreq,iRad)=(n-1)*(n-2)*...%2nd derivative
                    jnhn(iSubs,:,1,n,iFreq,iRad)./(Kpa.*Kpa)-...
                    jnhn(iSubs,:,1,n,iFreq,iRad)+...
                    2*jnhn(iSubs,:,1,n+1,iFreq,iRad)./Kpa; 
    
            end
            %now calc derivative for the highest order (stored in index nMax+1)
            jnhn(iSubs,:,2,nMax+1,iFreq,iRad)=(nMax)*...
                jnhn(iSubs,:,1,nMax+1,iFreq,iRad)./Kpa...
                -besselj(nMax+1.5,Kpa)*spherFactor./rootKpa;
            jnhn(iSubs,:,3,nMax+1,iFreq,iRad)=nMax*(nMax-1)*...%2nd derivative
                jnhn(iSubs,:,1,nMax+1,iFreq,iRad)./(Kpa.*Kpa)-...
                jnhn(iSubs,:,1,nMax+1,iFreq,iRad)+...
                2*besselj(nMax+1.5,Kpa)*spherFactor./(rootKpa.*Kpa);
       end
   end
end
