function [DmultScatt] = DmultScattmultSpecies(nMax,Kaa)
%This is the function D(x) defined for multiple scattering in a multiple
%species system (acoustic only, no mode conversion)
%A.L. Gower, M.J.A. Smith, W.J. Parnell, I.D. Abrahams, 
%Reflection from a multi-species material and its transmitted effective wavenumber, 
%Proc. R. Soc. A Math. Phys. Eng. Sci. 474 (2018) 20170864. 
%https://doi.org/10.1098/rspa.2017.0864.
%Supplementary materials after Eq 11
%which is identical to Linton Martin 2006 Eq 4.23
%**********
%calculated for all orders up to 2*nMax (which is the max q value in the summation)******
%********************
%It expresses the effect of excluded volume between 2 similar or dissimilar
%scatterers, and the combinations of Bessel and Hankel functsion arising
%from multiple scattering 
%Input:
%nMax: the maximum partial wave order required
%Kaa is the K of the continuous phase (at each frequency) multiplied by the
%combined radius of the two scatterers.
%this combined radius aa may be defined in different ways but the simplest 
% is aa=(ai+aj) where ai and aj are radius of the two scatterers i,j
%There is a set of all different combinations of i,j
%So Kaa is an array (numFreq,numPairs) 
%where numFreq is the number of frequencies in the set
%and numPairs is the number of different dispersed phase species combinations
%(including each size bin as a separate species)
%NOTE: ensure Kaa is this shape even when numSpecies=1
%Output:
%DmultScatt(2*nMax+1,numFreq,numPairs) is the value of D 
%for each partial wave order, frequency
%and each combination of scatterer species (size and/or type)


%Feb 2021
%pass in Ka(freq,numPairs) as K*combined radius
%D(2*nMax+1,numFreq,numPairs)
numFreq=size(Kaa,1);
numPairs=size(Kaa,2); %number of different scatterer types (sizes and/or types)

spherFactor=sqrt(pi/2); % to convert Jn to spherical bessel function jn
rootKaa=sqrt(Kaa);

%construct the set of Bessel and Hankel functions and derivatives
%jn(nOrder,ifreq,iSpecies,jSpecies)
jn=zeros(2*nMax+1,numFreq,numPairs);
hn=zeros(2*nMax+1,numFreq,numPairs);
jnp=zeros(2*nMax+1,numFreq,numPairs);
hnp=zeros(2*nMax+1,numFreq,numPairs);

DmultScatt=zeros(2*nMax+1,numFreq,numPairs);

%zero order function, n=0 stored in n+1 array element
jn(1,:,:)=sin(Kaa)./Kaa;
hn(1,:,:)=-1i*exp(1i*Kaa)./Kaa;

for n=1:2*nMax    %do each order : calc derivative on next loop
    jn(n+1,:,:)=besselj(n+0.5,Kaa)...
                    *spherFactor./rootKaa; %jn 
    hn(n+1,:,:)=besselh(n+0.5,1,Kaa)*...
                spherFactor./rootKaa; %hn 
    %now calculate derivatives for order n-1 from the recurrence relation
    jnp(n,:,:)=(n-1)*shiftdim(jn(n,:,:),1)./Kaa-shiftdim(jn(n+1,:,:),1);
    hnp(n,:,:)=(n-1)*shiftdim(hn(n,:,:),1)./Kaa-shiftdim(hn(n+1,:,:),1);
end
%finish with the derivative for the order nMax from recurrence relation
% using bessel and hankel for the partial wave order nMax+1
jnp(2*nMax+1,:,:)=(2*nMax)*shiftdim(jn(2*nMax+1,:,:),1)./Kaa-besselj(2*nMax+1.5,Kaa)...
                    *spherFactor./rootKaa;
hnp(2*nMax+1,:,:)=(2*nMax)*shiftdim(hn(2*nMax+1,:,:),1)./Kaa-besselh(2*nMax+1.5,1,Kaa)*...
                spherFactor./rootKaa; 

%Now calculate the multiple scattering function D 


for n=0:2*nMax
    DmultScatt(n+1,:,:)=Kaa.*shiftdim(jnp(n+1,:,:),1)...
        .*(Kaa.*shiftdim(hnp(n+1,:,:),1)+shiftdim(hn(n+1,:,:),1))+...
        (Kaa.*Kaa-n*(n+1)).*shiftdim(jn(n+1,:,:),1).*shiftdim(hn(n+1,:,:),1);
    %corrected 17/2/21 the last term to be hn 
    %due to typo in Gower2018 paper (compare with
    %Linton/Martin 2006 formula for d_m(x))
end

end