'''Copyright Tony Valier-Brasier 2024
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License
along with this program. If not, see https://www.gnu.org/licenses/.'''

import numpy as np
import h5py as h5
from matplotlib import pyplot as plt
import ScatteringCoefficients as fc_CD
import MST
import Functions
params = {'mathtext.fontset': 'cm', 'mathtext.rm': 'serif', 'font.size' : 15, 'legend.fontsize': 15,
          'xtick.labelsize': 15, 'ytick.labelsize': 15}
plt.rcParams.update(params)


''' Elastic parameters '''
R = 500e-9 # Sphere radius
f = 10**np.linspace(0,2.2,201)*1e6 # Frequency
mat_m = Functions.Materiau('eau',f) # Parameters of water
mat_p = Functions.Materiau('silica')# Parameters of spheres
kL = 2*np.pi*f*np.sqrt(mat_m.rho/(mat_m.lam + 2*mat_m.mu)) # Longitudinal wavenumber in water
kT = 2*np.pi*f*np.sqrt(mat_m.rho/mat_m.mu) # Shear wavenumber in water
phi = 0.297 # Concentration

''' Pair correlation function '''
data_file = h5.File('Spheres_phi29.7_a500nm.h5', 'r')
r = data_file['r'][()]
g2 = data_file['g2_mean'][()]
data_file.close()

''' Experimental results '''
fexp,Aexp = np.loadtxt('Attenuation_phi29.7_a500nm.csv', delimiter=';', usecols=(0,1), skiprows=2, unpack=True)


''' Calculation of the effective wavenumbers
Tn : scattering coefficients
K_MST : effective wavenumber with or without conversion (conv) and without correlation (corr)

'''
K_MST = np.zeros((3,np.size(f)), dtype = np.complex128)
Tn = fc_CD.Solide_Sphere_elastique(2, f, R, mat_p.lam, mat_p.mu, mat_p.rho, mat_m.lam, mat_m.mu, mat_m.rho)
K_MST[0,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'no',corr = 'no')
K_MST[1,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'no')
K_MST[2,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'yes',r=r,g2=g2)

''' PLOTS OF THE EFFECTIVE ATTENUATION '''
Attenuation = np.imag(K_MST)
lab = ['Model A', 'Model C', 'Model I']
col = ['r-.', 'C0--', 'k']
freq = np.real(kT*R)
freq_exp = R*np.real(np.sqrt(np.pi*fexp*mat_m.rho/mat_m.eta_s))*1e3
plt.figure(figsize = (8,5))
for nn in range(3):
    plt.plot(np.real(kT*R),Attenuation[nn,:]*mat_m.c0/f, col[nn], label = lab[nn])
plt.scatter(freq_exp,Aexp*mat_m.c0/fexp/1e6, marker = 'x', color = 'k', label = 'Exp')
plt.xscale('log')
plt.legend(frameon = False)
plt.xlim(1,10)
plt.xlabel('Re[$k_Sa$]')
plt.ylabel('$\\alpha_{eff}\\lambda$')
plt.tight_layout()
plt.savefig('Figure6.pdf', transparent = 'true', bbox_inches='tight')
plt.show()
