'''Copyright Tony Valier-Brasier 2024
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License
along with this program. If not, see https://www.gnu.org/licenses/.'''

import numpy as np
import h5py as h5
from matplotlib import pyplot as plt
import ScatteringCoefficients as fc_CD
import MST
import Functions
params = {'mathtext.fontset': 'cm', 'mathtext.rm': 'serif', 'font.size' : 15, 'legend.fontsize': 15,
          'xtick.labelsize': 15, 'ytick.labelsize': 15}
plt.rcParams.update(params)


''' Elastic parameters '''
R1 = 500e-9 # Sphere radius
R3 = 214e-9 # Sphere radius
f = 10**np.linspace(0,2.2,201)*1e6 # Frequency
mat_m = Functions.Materiau('eau',f) # Parameters of water
mat1_p = Functions.Materiau('silica')# Parameters of spheres
mat3_p = Functions.Materiau('silica_430')# Parameters of spheres
kL = 2*np.pi*f*np.sqrt(mat_m.rho/(mat_m.lam + 2*mat_m.mu)) # Longitudinal wavenumber in water
kT = 2*np.pi*f*np.sqrt(mat_m.rho/mat_m.mu) # Shear wavenumber in water

''' Pair correlation function '''
data_file = h5.File('Spheres_phi39.4_a500nm.h5', 'r')
r1 = data_file['r'][()]
g1 = data_file['g2_mean'][()]
data_file.close()
data_file = h5.File('Spheres_phi29.7_a500nm.h5', 'r')
r2 = data_file['r'][()]
g2 = data_file['g2_mean'][()]
data_file.close()
data_file = h5.File('Spheres_phi29_a215nm.h5', 'r')
r3 = data_file['r'][()]
g3 = data_file['g2_mean'][()]
data_file.close()

''' Experimental results '''
fexp1,Aexp1 = np.loadtxt('Attenuation_phi39.4_a500nm.csv', delimiter=';', usecols=(0,1), skiprows=2, unpack=True)
fexp2,Aexp2 = np.loadtxt('Attenuation_phi29.7_a500nm.csv', delimiter=';', usecols=(0,1), skiprows=2, unpack=True)
fexp3,Aexp3 = np.loadtxt('Attenuation_phi29_a215nm.csv', delimiter=';', usecols=(0,1), skiprows=2, unpack=True)


''' Calculation of the effective wavenumbers
Tn : scattering coefficients
K_MST : effective wavenumber with or without conversion (conv) and without correlation (corr)

'''
K_MST = np.zeros((3,np.size(f)), dtype = np.complex128)
Tn1 = fc_CD.Solide_Sphere_elastique(2, f, R1, mat1_p.lam, mat1_p.mu, mat1_p.rho, mat_m.lam, mat_m.mu, mat_m.rho)
Tn3 = fc_CD.Solide_Sphere_elastique(2, f, R3, mat3_p.lam, mat3_p.mu, mat3_p.rho, mat_m.lam, mat_m.mu, mat_m.rho)

phi = 0.394 # Concentration
K_MST[0,:] = MST.MST_keff(phi,R1,kL,kT,Tn1,2*R1,'yes','yes',r=r1,g2=g1)

phi = 0.297 # Concentration
K_MST[1,:] = MST.MST_keff(phi,R1,kL,kT,Tn1,2*R1,'yes','yes',r=r2,g2=g2)

phi = 0.29 # Concentration
K_MST[2,:] = MST.MST_keff(phi,R3,kL,kT,Tn3,2*R3,'yes','yes',r=r3,g2=g3)


''' PLOTS OF THE EFFECTIVE ATTENUATION '''
Attenuation = np.imag(K_MST)
lab = ['$\phi = 39.4\%$ (Model I)', '$\phi = 39.4\%$ (Exp)', '$\phi = 29.7\%$ (Model I)', '$\phi = 29.7\%$ (Exp)', '$\phi = 29\%$ (Model I)', '$\phi = 29\%$ (Exp)']
col = ['C0', 'k', 'r']
freq_exp1 = R1*np.real(np.sqrt(np.pi*fexp1*mat_m.rho/mat_m.eta_s))*1e3
freq_exp2 = R1*np.real(np.sqrt(np.pi*fexp2*mat_m.rho/mat_m.eta_s))*1e3
freq_exp3 = R3*np.real(np.sqrt(np.pi*fexp3*mat_m.rho/mat_m.eta_s))*1e3
plt.figure(figsize = (11,5))
plt.plot(np.real(kT*R1),Attenuation[0,:]*mat_m.c0/f, col[0], label = lab[0])
plt.scatter(freq_exp1,Aexp1*mat_m.c0/fexp1/1e6, marker = 'o', color = 'C0', label = lab[1])
plt.plot(np.real(kT*R1),Attenuation[1,:]*mat_m.c0/f, col[1], label = lab[2])
plt.scatter(freq_exp2,Aexp2*mat_m.c0/fexp2/1e6, marker = 'x', color = 'k', label = lab[3])
plt.plot(np.real(kT*R3),Attenuation[2,:]*mat_m.c0/f, col[2], label = lab[4])
plt.scatter(freq_exp3,Aexp3*mat_m.c0/fexp3/1e6, marker = '+', color = 'r', label = lab[5])
plt.xscale('log')
plt.legend(frameon = False, ncol = 3)
plt.xlim(1,10)
plt.ylim(0,0.1)
plt.xlabel('Re[$k_Sa$]')
plt.ylabel('$\\alpha_{eff}\\lambda$')
plt.tight_layout()
plt.savefig('Figure7.pdf', transparent = 'true', bbox_inches='tight')
plt.show()
