function [jn1rs,hn1rs,jn2rs,jn2rc_rs,jnp2rc_rs,hn2rc,hn2rs_rc,hnp2rs_rc,jn3rc,limImkhTF]=...
    besselcoreshell(nMax,K,freq,radius,shellThick)
%for use with core-shell ultrasound scattering calculations based on
%Anson/Chivers model
%calculate the Bessel functions jn and hn and derivatives up to the max order
%also the jn and its derivative for the incoming wave
%
%inputs
%nMax: the maximum order needed,
%K(iSubs,iMode,iFreq): the wavenumbers for all substances, modes, and frequencies
%freq: the frequency values
%radius: radius of the core (could be a vector of values) metres
%shellThick: shell thickness (metres)
%
%Define rc=inner radius (core), rs=outer radius (shell)
%outputs:
%Outer scaled by value at shell bdy, core scaled by value at core/shell
%bdry
%Shell: jn scaled by value at shell bdry, hn scale dby value at core bdry
%*** This is different from A&C but avoid problems of large exponential
%terms when Im(k) is large in shell.
%
%jn1rs(n,iFreq,iRad):=jn(kc1rs)
%all others have indices (mode,order+1,iFreq,iRad)
%hn1rs: =hn+1(k1rs)/hn(k1rs) each mode
%jn2rs=jn+1(k2rs)/jn(k2rs) each mode,
%jn2rc_rs=jn(k2rc)/jn(k2rs)
%jnp2rc_rs=jn+1(k2rc)/jn(k2rs),
%%hn2rc=hn+1(k2rc)/hn(k2rc) each mode,
%hn2rs_rc=hn(k2rs)/hn(k2rc)
%hnp2rs_rc=hn+1(k2rs)/hn(k2rc),
%jn3rc=jn+1(k3rc)/jn(k3rc) %This may be different scaling from Anson/Chivers 
%limImkhTF is the true/false flag for Im(k2)*(b-a)>limImkh=10,000 (k any
%mode)
%limImkhTF(iMode,iFreq,iRad)

%x refers to K*rs (outer boundary), y refers to K*rc (inner boundary)
%
limImkh=100; %the limit of |k(a-b)| above which some further scaling of 
%terms in the matrix boundary equation is needed.
%Use the scaled Bessel and Hankel function outputs from Matlab functions
numFreq=size(freq,2);
numRad=size(radius,2);
spherFactor=sqrt(pi/2); % to convert Jn to spherical bessel function jn
nMax=nMax+1; %need to calculate one extra
%Create arrays
jn1rs=zeros(nMax+2,numFreq,numRad);
%all others have indices (mode,n,iFreq,iRad)
hn1rs=zeros(3,nMax+1,numFreq,numRad);
jn2rs=zeros(3,nMax+1,numFreq,numRad);
jn2rc_rs=zeros(3,nMax+1,numFreq,numRad);
jnp2rc_rs=zeros(3,nMax+1,numFreq,numRad);
hn2rc=zeros(3,nMax+1,numFreq,numRad);
hn2rs_rc=zeros(3,nMax+1,numFreq,numRad);
hnp2rs_rc=zeros(3,nMax+1,numFreq,numRad);
jn3rc=zeros(3,nMax+1,numFreq,numRad);

numFR=numFreq*numRad; %number of combinations of radius and frequency
ordersMode1=repmat((0.5:nMax+1+0.5)',1,numFR); %create orders as a column repeated numFreq*numRad times
outDimsMode1=[nMax+2,numFreq,numRad]; %output from Bessel function calcns for single mode
orders=repmat((0.5:nMax+1+0.5)',1,3*numFR); %create orders as a column repeated numFreq*numRad times
outDims=[nMax+2,3,numFreq,numRad];%output from Bessel function calcns for all modes
%create arrays of wavenumber-radius product at inner and outer radii
%KLSa(iSubs,iMode,iFReq,iRad)
radius=radius(:);%make it into a column
%modified with[] around dimensions 17/6/2015 based on Matlab warning about
%future release change to repmat - can't use row vectors
KLSrc=shiftdim(repmat(radius, [1,size(K)]),1).*repmat(K,[[1 1 1],numRad]);
KLSrs=shiftdim(repmat(radius+shellThick,[1,size(K)]),1).*repmat(K,[[1 1 1],numRad]);

%dispersed phase: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%stores only Bessel function at outer radius
iSubs=1;
%mode 1 compressional mode only for incident wave
bessArgs=repmat(reshape(KLSrs(iSubs,1,:,:),[1,numFreq*numRad]),nMax+2,1); %Krs values
sqrtbessArgs=sqrt(bessArgs);
%store only the Bessel function itself for incident wave
jn1rs=spherFactor*reshape((besselj(ordersMode1,bessArgs,1)./sqrtbessArgs),outDimsMode1);

%all modes for scattered waves in dispersed phase
bessArgs=repmat(reshape(KLSrs(iSubs,:,:,:),[1,3*numFreq*numRad]),nMax+2,1); %Krs(=x) values
sqrtbessArgs=sqrt(bessArgs);
%note that 'orders' is first dimensions of bessel output - then permute
%dimensions to get mode, order, freq,rad
hn=spherFactor*permute(reshape((besselh(orders,1,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]);
%scale the result
for n=0:nMax %all orders
    hn1rs(:,n+1,:,:)=hn(:,n+2,:,:)./hn(:,n+1,:,:); %=hn+1(krs)/hn(krs)
end

%core%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%core: stores only Bessel functions at inner radius, all modes
iSubs=3;
bessArgs=repmat(reshape(KLSrc(iSubs,:,:,:),[1,3*numFreq*numRad]),nMax+2,1); %Krc (=y) values
sqrtbessArgs=sqrt(bessArgs);
%note that 'orders' is first dimensions of bessel output - then permute
%dimensions to get mode, order, freq,rad
jn=spherFactor*permute(reshape((besselj(orders,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]);
%scale the result
for n=0:nMax %all orders
    jn3rc(:,n+1,:,:)=jn(:,n+2,:,:)./jn(:,n+1,:,:); %jn+1(krc)/jn(krc)
end

%shell%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%shell: stores Bessel functions at inner and outer radius, all modes
%scaled as per definitions above (by functions at outer radius)
%outer radii functions first
iSubs=2;
bessArgs=repmat(reshape(KLSrs(iSubs,:,:,:),[1,3*numFreq*numRad]),nMax+2,1); %Krs (=x) values
sqrtbessArgs=sqrt(bessArgs);
%note that 'orders' is first dimensions of bessel output - then permute
%dimensions to get mode, order, freq,rad
hnx=spherFactor*permute(reshape((besselh(orders,1,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]); %hn(krs)
jnx=spherFactor*permute(reshape((besselj(orders,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]); %jn(krs)
bessArgs=repmat(reshape(KLSrc(iSubs,:,:,:),[1,3*numFreq*numRad]),nMax+2,1); %Krc (=y) values
sqrtbessArgs=sqrt(bessArgs);
%note that 'orders' is first dimensions of bessel output - then permute
%dimensions to get mode, order, freq,rad
hny=spherFactor*permute(reshape((besselh(orders,1,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]); %hn(krc)
jny=spherFactor*permute(reshape((besselj(orders,bessArgs,1)./sqrtbessArgs),...
    outDims),[2 1 3 4]); %jn(krc)

%scale the results
%calculate K*shell thickness; shift dimensions so subs is at end to avoid
%problems when nFreq=nRad=1 (which would produce different shape array than
%the Bessel function arrays at a single order)
KLSrc_rs=shiftdim((-KLSrs(2,:,:,:)+KLSrc(2,:,:,:)),1); %calculate K*shell thickness=K(rc-rs)<0
limImkhTF=squeeze(imag(-KLSrc_rs)>limImkh); %is the imag(K)*(shell thickness) v. large?

%if so, set Bessel function ratios jn(y)/jn(x) and jn+1(y)/jn(x) to zero
%and the Hankel function ratios hn(x)/hn(y) hn+1(x)/hn(y) to zero
%scalings for hn different from A&C so both Bessel and Hankel ratios 
%(at different radii) tend
%to zero when large Im(k)
for n=0:nMax %all orders
    jn2rs(:,n+1,:,:)=jnx(:,n+2,:,:)./jnx(:,n+1,:,:); %jn+1/jn
    hn2rc(:,n+1,:,:)=hny(:,n+2,:,:)./hny(:,n+1,:,:); %hn+1/hn at rc
    %inner/outer scaling
    %using scaled Bessel/Hankel functions, so need to include the missed
    %terms in ratios of functions at different arguments
    %have to check the limit on Imag(k)*shell thickness for each condition
    for iMode=1:3
        for iFreq=1:numFreq
            for iRad=1:numRad
                if limImkhTF(iMode,iFreq,iRad) %oversize on Im(k)*shellthickness
                    jn2rc_rs(iMode,n+1,iFreq,iRad)=0; %jn(y=Krc)/jn(x=Krs) is v small
                    jnp2rc_rs(iMode,n+1,iFreq,iRad)=0; %jn+1(y=Krc)/jn(x=Krs) is v small
                    hn2rs_rc(iMode,n+1,iFreq,iRad)=0; %hn(x=Krs)/hn(y=Krc) is v small %NOTE different scaling from A&C
                    hnp2rs_rc(iMode,n+1,iFreq,iRad)=0;%hn=1(x=Krs)/hn(y=Krc) is v small
                else %within the limit for Im(k)*shellthickness. 
                    %Normal calculations with Bessel/Hankel scaling factors put in 
                    %(because used Matlab scaled functions to remove exp(Im(K) factors)
                    
                    jn2rc_rs(iMode,n+1,iFreq,iRad)=squeeze(jny(iMode,n+1,iFreq,iRad)).*...
                        exp(squeeze(imag(KLSrc_rs(iMode,iFreq,iRad))))...
                    ./squeeze(jnx(iMode,n+1,iFreq,iRad)); %jn(y=Krc)/jn(x=Krs)
                    jnp2rc_rs(iMode,n+1,iFreq,iRad)=squeeze(jny(iMode,n+2,iFreq,iRad))...
                    .*exp(squeeze(imag(KLSrc_rs(iMode,iFreq,iRad))))...
                    ./squeeze(jnx(iMode,n+1,iFreq,iRad)); %jn+1(y=Krc)/jn(x=Krs) 
                    hn2rs_rc(iMode,n+1,iFreq,iRad)=squeeze(hnx(iMode,n+1,iFreq,iRad))...
                    .*exp(-1i*squeeze(KLSrc_rs(iMode,iFreq,iRad)))...
                    ./squeeze(hny(iMode,n+1,iFreq,iRad)); %hn(x=Krs)/jn(y=Krc)
                    hnp2rs_rc(iMode,n+1,iFreq,iRad)=squeeze(hnx(iMode,n+2,iFreq,iRad))...
                    .*exp(-1i*squeeze(KLSrc_rs(iMode,iFreq,iRad)))...
                    ./squeeze(hny(iMode,n+1,iFreq,iRad)); %hn+1(x=Krs)/hn(y=Krc)
                 end
            end
        end
    end
end
if sum(sum(sum(isnan(jn1rs),1),2)) || sum(sum(sum(sum(isnan(jn2rs),1),2),3))||...
sum(sum(sum(sum(isnan(jn2rc_rs),1),2),3))||...
sum(sum(sum(sum(isnan(jnp2rc_rs),1),2),3))||...
sum(sum(sum(sum(sum(isnan(jn3rc),1),2))))||...
sum(sum(sum(sum(isnan(hn1rs),1),2),3))||...
sum(sum(sum(sum(isnan(hn2rs_rc),1),2),3))||...
sum(sum(sum(sum(isnan(hnp2rs_rc),1),2),3))
warning('Some Bessel or Hankel functions stored Nans')
end