function [scattCoeffn,scattCoeffPn,matrixNearSing,nMax_OK]=...
    scattCoeffACShell...
    (nMax,propsCont,propsShell,propsDisp,f,rc,rs,...
    jn1rs,hn1rs,jn2rs,jn2rc_rs,jnp2rc_rs,hn2rc,hn2rs_rc,hnp2rs_rc,jn3rc,limImkhTF,...
    K1,L1,S1,K2,L2,S2,K3,L3,S3,...
    gammaC1,gammaT1,gammaC2,gammaT2,gammaC3,gammaT3,ECAHOption)
%Core-shell model of Anson/Chivers
%solutions for single subst, freq and size
%calculate the scattering coefficients An etc
%Initial code by Graham Boyle 03/07/2013
%Corrected and edited for scaling of Bessel functions by Valerie Pinfield Feb 2014
%References:
%Yinghui Chen, PhD thesis (2007)
%Anson&Chivers (1993)
%Inputs: 
%nMax:maximum order n, calculates to nMax+1 and array index is (n+1)
%propsCont: structure for properties of continuous phase,with (at least) field 'density','shearViscMod','conduct','matType'
%propsShell: as above for shell phase
%propsDisp: as above for disp phase
%f:freq in MHz
%a: core radius in metres
%b: shell outer radius in metres
%For the following variables 1=cont phase, 2=shell phase, 3=disp phase
%Outside this function, arrays are (with freq and radius dimensions removed
%here)
%jn1rs(n+1):=jn(kc1rs)
%all others have indices (mode,order+1) here
%hn1rs: =hn+1(k1rs)/hn(k1rs) each mode
%jn2rs=jn+1(k2rs)/jn(k2rs) each mode,
%jn2rc_rs=jn(k2rc)/jn(k2rs)
%jnp2rc_rs=jn+1(k2rc)/jn(k2rs),
%%hn2rc=hn+1(k2rc)/hn(k2rc) each mode, %Different scaling from
%%Anson/Chivers
%hn2rs_rc=hn(k2rs)/hn(k2rc)%Different scaling from
%%Anson/Chivers
%hnp2rs_rc=hn+1(k2rs)/hn(k2rc),%Different scaling from
%%Anson/Chivers
%jn3rc=jn+1(k3rc)/jn(k3rc) 
%
%here jn1b(n+1) and all others(mode, order+1)
%
%limImkhTF outside this function is (mode,freq,rad), here (mode)
%true/false as to whether Im(k in subs 2)*shellthickness>10,000
%K,L,S:  wavenumbers of comp, thermal, shear modes for phases 1,2,3
%gammaC,T: thermal factors for comp, thermal, shear modes, in phase 1,2,3
%Outputs:
%An,Bn,Cn=scattCoeffn(mode 1:3,n+1)Outgoing (hn) coeffs in cont phase
%Gn,Hn,In=scattCoeffPn(mode 1:3,n+1)Outgoing (hn) coeffs in shell
%Note these are incorrectly identified in Anson and Chivers 
%(the matrix equation lists the coefficients in alphabetical order, 
%but in fact the matrix is constructed with all jn terms first)
%they were redefined by Yinghui Chen, and used as YC here
%the output coefficients are for the outgoing modes in the shell

warning('off','MATLAB:nearlySingularMatrix'); %turn off warnings about near-singular matrices
noShear=0; %set to 1 to omit shear waves everywhere
noTherm=0; %set to 1 to omit thermal waves everywhere 

persistent alreadyAskednMax %flag whether user has confirmed nMax too small but continuing
if isempty(alreadyAskednMax)
    alreadyAskednMax=false; %initalise it on the first function call
end
nMax_OK=true;
matrixNearSing=false; %set false flag for this call to function

spherFactor=sqrt(pi/2); % to convert Jn to spherical bessel function jn
        
scattCoeffn=zeros(3,nMax+2);
scattCoeffPn=zeros(3,nMax+2);

K1rs=K1*rs; %(previously 'Ka') in Anson & Chivers notation: xd1
L1rs=L1*rs; %(previously 'La') in Anson & Chivers notation: xt1
S1rs=S1*rs; %(previously 'Sa') in Anson & Chivers notation: xs1
K2rs=K2*rs; %in Anson & Chivers notation: xd2
L2rs=L2*rs; %in Anson & Chivers notation: xd2
S2rs=S2*rs; %in Anson & Chivers notation: xd2
K2rc=K2*rc; %in Anson & Chivers notation: yd2
L2rc=L2*rc; %in Anson & Chivers notation: yt2
S2rc=S2*rc; %in Anson & Chivers notation: ys2
K3rc=K3*rc; %(previously 'Kpa') in Anson & Chivers notation: yd3
L3rc=L3*rc; %(previously 'Lpa') in Anson & Chivers notation: yt3
S3rc=S3*rc; %(previously 'Spa') in Anson & Chivers notation: ys3

nLast=nMax+1;

%Matrix Calculation
for n=0:nLast %each order
    
    % Simplifications:
    nn_1=n*(n+1);
    %n2n=n*n+n-2;
    LHS=zeros(12,12);
    RHS=zeros(12,1);
    coeffs=zeros(1,12);
    TX1=1-((2*n*(n-1))/(S1rs^2)); %taken from Anson&Chivers pg. 1689 
    TX2=1-((2*n*(n-1))/(S2rs^2));
    TY2=1-((2*n*(n-1))/(S2rc^2));
    TY3=1-((2*n*(n-1))/(S3rc^2));
    densityRatio12=propsCont.density/propsShell.density;
    densityRatio32=propsDisp.density/propsShell.density;
    conductRatio21=propsShell.conduct/propsCont.conduct;
    conductRatio23=propsShell.conduct/propsDisp.conduct;
    
    %LHS
    %Column 1: (scaled by hn(Krs1)
    LHS(1,1)= n-(K1rs*hn1rs(1,n+1));
    LHS(2,1)= 1;
    LHS(3,1)= (gammaC1/gammaC2);
    LHS(4,1)= (gammaC1/gammaC2)*LHS(1,1);
    LHS(5,1)= densityRatio12*(TX1-(4*K1rs*hn1rs(1,n+1))/S1rs^2);
    LHS(6,1)= (1/S1rs^2)*((n-1)-K1rs*hn1rs(1,n+1))*densityRatio12;
    
    %Column 2: (divided by L1b^3, scaled by hn(Lb1)
    LHS(1,2)= (1/L1rs^3)*(n-L1rs*hn1rs(2,n+1));
    LHS(2,2)= (1/L1rs^3);
    LHS(3,2)= (1/L1rs^3)*(gammaT1/gammaC2);
    LHS(4,2)= (1/L1rs^3)*(gammaT1/gammaC2)*(n-L1rs*hn1rs(2,n+1));
    LHS(5,2)= (1/L1rs^3)*densityRatio12*(TX1-4*L1rs*hn1rs(2,n+1)/S1rs^2);
    LHS(6,2)= (1/L1rs^3)*(1/S1rs^2)*((n-1)-L1rs*hn1rs(2,n+1))*densityRatio12;
    
    %Column 3: (divided by S1b=xs1,scaled by hn(Sb1)=hn(xs1)
    LHS(1,3)= -(1/S1rs)*nn_1;
    LHS(2,3)= -(1/S1rs)*((n+1)-S1rs*hn1rs(3,n+1));
    LHS(5,3)= (1/S1rs^3)*densityRatio12*2*nn_1*((n-1)-S1rs*hn1rs(3,n+1));
    LHS(6,3)= -(1/S1rs^3)*densityRatio12*((n^2-1-(S1rs^2)/2)+S1rs*hn1rs(3,n+1));
    
    %Column 4:scaled by jn(Kc2rs)=jn(xc2)
    LHS(1,4)= n-K2rs*jn2rs(1,n+1);
    LHS(2,4)= 1.0;
    LHS(3,4)= 1.0;
    LHS(4,4)= conductRatio21*(n-K2rs*jn2rs(1,n+1));
    LHS(5,4)= TX2-(4*K2rs*jn2rs(1,n+1))/S2rs^2;
    LHS(6,4)= (1/S2rs^2)*((n-1)-K2rs*jn2rs(1,n+1));
    LHS(7,4)= n*jn2rc_rs(1,n+1)-K2rc*jnp2rc_rs(1,n+1);
    LHS(8,4)= jn2rc_rs(1,n+1);
    LHS(9,4)= jn2rc_rs(1,n+1);
    LHS(10,4)= conductRatio23*(n*jn2rc_rs(1,n+1)-K2rc*jnp2rc_rs(1,n+1));
    LHS(11,4)= TY2*jn2rc_rs(1,n+1)-(4*K2rc*jnp2rc_rs(1,n+1)/S2rc^2);
    LHS(12,4)= (1/S2rc^2)*((n-1)*jn2rc_rs(1,n+1)-K2rc*jnp2rc_rs(1,n+1));
        
    %Column 5 (divided by L2b^3):scaled by jn(L2rs)=jn(xt2)
    LHS(1,5)= (1/L2rs^3)*(n-L2rs*jn2rs(2,n+1));
    LHS(2,5)= (1/L2rs^3);
    LHS(3,5)= (1/L2rs^3)*(gammaT2/gammaC2);
    LHS(4,5)= (1/L2rs^3)*conductRatio21*(gammaT2/gammaC2)*(n-L2rs*jn2rs(2,n+1));
    LHS(5,5)= (1/L2rs^3)*(TX2-4*L2rs*jn2rs(2,n+1)/S2rs^2);
    LHS(6,5)= (1/L2rs^3)*(1/S2rs^2)*((n-1)-L2rs*jn2rs(2,n+1));
    LHS(7,5)= (1/L2rs^3)*(n*jn2rc_rs(2,n+1)-L2rc*jnp2rc_rs(2,n+1)); %note (b/a) in Anson & Chivers (opposite convention)
    LHS(8,5)= (1/L2rs^3)*jn2rc_rs(2,n+1);
    LHS(9,5)= (1/L2rs^3)*(gammaT2/gammaC2)*jn2rc_rs(2,n+1);
    LHS(10,5)= (1/L2rs^3)*(gammaT2/gammaC2)*conductRatio23*(n*jn2rc_rs(2,n+1)-L2rc*jnp2rc_rs(2,n+1));
    LHS(11,5)= (1/L2rs^3)*(TY2*jn2rc_rs(2,n+1)-4*L2rc*jnp2rc_rs(2,n+1)/S2rc^2);
    LHS(12,5)= (1/L2rs^3)*(1/S2rc^2)*((n-1)*jn2rc_rs(2,n+1)-L2rc*jnp2rc_rs(2,n+1));
    
    %Column 6 (divided by S2b):scaled by jn(Sb2)=jn(xs2)
    LHS(1,6)= -(1/S2rs)*nn_1;
    LHS(2,6)= -(1/S2rs)*((n+1)-S2rs*jn2rs(3,n+1));
    LHS(5,6)= (1/S2rs^3)*2*nn_1*((n-1)-S2rs*jn2rs(3,n+1)); %typo in Phd thesis uses jn1b instead of jn2b
    LHS(6,6)= -(1/S2rs^3)*((n^2-1-(S2rs^2/2))+S2rs*jn2rs(3,n+1));
    LHS(7,6)= -(1/S2rs)*nn_1*jn2rc_rs(3,n+1);
    LHS(8,6)= -(1/S2rs)*((n+1)*jn2rc_rs(3,n+1)-S2rc*jnp2rc_rs(3,n+1));
    LHS(11,6)= (1/S2rs)*(1/S2rc^2)*2*nn_1*((n-1)*jn2rc_rs(3,n+1)-S2rc*jnp2rc_rs(3,n+1));
    LHS(12,6)= -(1/S2rs)*(1/S2rc^2)*((n^2-1-(S2rc^2/2))*jn2rc_rs(3,n+1)+S2rc*jnp2rc_rs(3,n+1));

    %columns 7-9 are the same as 4-6 repectively but with teh bessel functions
    %jn, replaced with hankel functions hn of the same order and argument
    %but scaling is different - using rc not rs
    %Column 7:scaled by hn(Kc2rc)=hn(xC2)
    LHS(1,7)= n*hn2rs_rc(1,n+1)-K2rs*hnp2rs_rc(1,n+1);
    LHS(2,7)= hn2rs_rc(1,n+1);
    LHS(3,7)= hn2rs_rc(1,n+1);
    LHS(4,7)= conductRatio21*(n*hn2rs_rc(1,n+1)-K2rs*hnp2rs_rc(1,n+1));
    LHS(5,7)= TX2*hn2rs_rc(1,n+1)-4*K2rs*hnp2rs_rc(1,n+1)/S2rs^2;
    LHS(6,7)= (1/S2rs^2)*((n-1)*hn2rs_rc(1,n+1)-K2rs*hnp2rs_rc(1,n+1));
    LHS(7,7)= n-K2rc*hn2rc(1,n+1);
    LHS(8,7)= 1.0;
    LHS(9,7)= 1.0;
    LHS(10,7)= conductRatio23*(n-K2rc*hn2rc(1,n+1));
    LHS(11,7)= TY2-4*K2rc*hn2rc(1,n+1)/S2rc^2;
    LHS(12,7)= (1/S2rc^2)*((n-1)-K2rc*hn2rc(1,n+1));
    
    %Column 8 (divided by L2b^3):scaled by jn(Lb2)=jn(xt2)
    LHS(1,8)= (1/L2rs^3)*(n*hn2rs_rc(2,n+1)-L2rs*hnp2rs_rc(2,n+1));
    LHS(2,8)= (1/L2rs^3)*hn2rs_rc(2,n+1);
    LHS(3,8)= (1/L2rs^3)*(gammaT2/gammaC2)*hn2rs_rc(2,n+1);
    LHS(4,8)= (1/L2rs^3)*conductRatio21*(gammaT2/gammaC2)*...
        (n*hn2rs_rc(2,n+1)-L2rs*hnp2rs_rc(2,n+1));
    LHS(5,8)= (1/L2rs^3)*(TX2*hn2rs_rc(2,n+1)-4*L2rs*hnp2rs_rc(2,n+1)/S2rs^2);
    LHS(6,8)= (1/L2rs^3)*(1/S2rs^2)*((n-1)*hn2rs_rc(2,n+1)-L2rs*hnp2rs_rc(2,n+1));
    LHS(7,8)= (1/L2rs^3)*(n-L2rc*hn2rc(2,n+1));
    LHS(8,8)= (1/L2rs^3);
    LHS(9,8)= (1/L2rs^3)*(gammaT2/gammaC2);
    LHS(10,8)= (1/L2rs^3)*(gammaT2/gammaC2)*conductRatio23*(n-L2rc*hn2rc(2,n+1));
    LHS(11,8)= (1/L2rs^3)*(TY2-4*L2rc*hn2rc(2,n+1)/S2rc^2);
    LHS(12,8)= (1/L2rs^3)*(1/S2rc^2)*((n-1)-L2rc*hn2rc(2,n+1));
    
      
    %Column 9 (divided by S2b):scaled by hn(Sb2)=hn(xs2)
    LHS(1,9)= -(1/S2rs)*nn_1*hn2rs_rc(3,n+1);
    LHS(2,9)= -(1/S2rs)*((n+1)*hn2rs_rc(3,n+1)-S2rs*hnp2rs_rc(3,n+1));
    LHS(5,9)= (1/S2rs^3)*2*nn_1*((n-1)*hn2rs_rc(3,n+1)-S2rs*hnp2rs_rc(3,n+1));
    LHS(6,9)= -(1/S2rs^3)*((n^2-1-(S2rs^2/2))*hn2rs_rc(3,n+1)+S2rs*hnp2rs_rc(3,n+1));
    LHS(7,9)= -(1/S2rs)*nn_1;
    LHS(8,9)= -(1/S2rs)*((n+1)-S2rc*hn2rc(3,n+1)); %typo in YC thesis
    LHS(11,9)= (1/S2rs)*(1/S2rc^2)*2*nn_1*((n-1)-...
        S2rc*hn2rc(3,n+1)); %typo in YC thesis (+ not-)
    LHS(12,9)= -(1/S2rs)*(1/S2rc^2)*((n^2-1-(S2rc^2/2))+...
        S2rc*hn2rc(3,n+1)); %typo in YC thesis
    
  
    
    %Column 10:scaled by jn(Ka3)=jn(yc3) %VP scaliing, AC use jn(xc3)
    LHS(7,10)= n-K3rc*jn3rc(1,n+1);
    LHS(8,10)= 1.0;
    LHS(9,10)= gammaC3/gammaC2;
    LHS(10,10)= (gammaC3/gammaC2)*(n-K3rc*jn3rc(1,n+1));
    LHS(11,10)= densityRatio32*(TY3-4*K3rc*jn3rc(1,n+1)/S3rc^2);
    LHS(12,10)= densityRatio32*(1/S3rc^2)*((n-1)-K3rc*jn3rc(1,n+1));
    
    %Column 11 (divided by L3a^3):scaled by jn(La3)=jn(yt3) %VP scaliing, AC use jn(xt3)
    LHS(7,11)= (1/L3rc^3)*(n-L3rc*jn3rc(2,n+1));
    LHS(8,11)= (1/L3rc^3);
    LHS(9,11)= (1/L3rc^3)*(gammaT3/gammaC2);
    LHS(10,11)= (1/L3rc^3)*(gammaT3/gammaC2)*(n-L3rc*jn3rc(2,n+1));
    LHS(11,11)= (1/L3rc^3)*densityRatio32*(TY3-4*L3rc*jn3rc(2,n+1)/S3rc^2);
    LHS(12,11)= (1/L3rc^3)*(1/S3rc^2)*densityRatio32*((n-1)-L3rc*jn3rc(2,n+1));
    
    %Column 12 (divided by S3a):scaled by jn(Sa3)=jn(yS3) %VP scaling, AC use jn(xS3)

    LHS(7,12)= -(1/S3rc)*nn_1;
    LHS(8,12)= -(1/S3rc)*((n+1)-S3rc*jn3rc(3,n+1));
    LHS(11,12)= (1/S3rc^3)*densityRatio32*2*nn_1*((n-1)-S3rc*jn3rc(3,n+1));
    LHS(12,12)= -(1/S3rc^3)*densityRatio32*((n^2-1-(S3rc^2/2))+...
        S3rc*jn3rc(3,n+1));
    
     
    %RHS: do not implement scaling. AC use hn(xc1)
    RHS(1,1)=-((n*jn1rs(n+1))-(K1rs*jn1rs(n+2)));
    RHS(2,1)=-jn1rs(n+1);
    RHS(3,1)=-(gammaC1/gammaC2)*jn1rs(n+1);
    RHS(4,1)=-(gammaC1/gammaC2)*((n*jn1rs(n+1))-(K1rs*jn1rs(n+2)));
    RHS(5,1)=-densityRatio12*((TX1*jn1rs(n+1))-(((4*K1rs)/S1rs^2)*jn1rs(n+2)));
    RHS(6,1)=-densityRatio12*(1/S1rs^2)*(((n-1)*jn1rs(n+1))-(K1rs*jn1rs(n+2)));
    
    %Matrix for comparrison to original ECAH
    %when the shell thickness=zero and the particle=core material
    LHS6_6=zeros(6,6);
    %{
    %This block uses the core material as the particle as a check of A&C
    LHS6_6(1,:)=[LHS(1,1),LHS(1,2),LHS(1,3),LHS(7,10),LHS(7,11),LHS(7,12)];
    LHS6_6(2,:)=[LHS(2,1),LHS(2,2),LHS(2,3),LHS(8,10),LHS(8,11),LHS(8,12)];
    LHS6_6(3,:)=[LHS(3,1),LHS(3,2),LHS(3,3),LHS(9,10),LHS(9,11),LHS(9,12)];
    LHS6_6(4,:)=[LHS(4,1),LHS(4,2),LHS(4,3),LHS(10,10),LHS(10,11),LHS(10,12)];
    LHS6_6(5,:)=[LHS(5,1),LHS(5,2),LHS(5,3),LHS(11,10),LHS(11,11),LHS(11,12)];
    LHS6_6(6,:)=[LHS(6,1),LHS(6,2),LHS(6,3),LHS(12,10),LHS(12,11),LHS(12,12)];
    RHS6_6=RHS(1:6);
    %}
    %Use the shell material for the particle for comparison with ECAH when
    %shell thickness=zero and particle=shell material
    LHS6_6=LHS(1:6,1:6);
    RHS6_6=RHS(1:6);
        
    if n==0 %do the n=0 solution, no shear, can simplify
      
       
        LHS8_8=LHS([1 3:5 7 9:11],[1:2 4:5 7:8 10:11]);  
        RHS8_8=RHS([1 3:5 7 9:11]);
        
        coeffs(1:8)=LHS8_8\RHS8_8;
       
        if ECAHOption
            %calculate just the ECAH solution with core material and zero
            %shell thickness
            LHS4_4=LHS6_6([1 3 4 5],[1 2 4 5]); %no shear waves, no shear conditions for n=0
            RHS4_4=RHS6_6([1 3 4 5]);
            coeffs([1 2 7 8])=LHS4_4\RHS4_4;
        end
        
      if noTherm   %if requested remove all thermal terms and bdry condns 
        newLHS=LHS([1 5 7 11],[1 4 7 10]);
        newRHS=RHS([1 5 7 11]);
        coeffsNew=newLHS\newRHS;
        coeffs(1)=coeffsNew(1);
      end
      %}
        %RESULTS for n=0
        %STORE the compressional mode UNSCALED for consistency with other
        %models and to make the multiple scattering calcn easier
        scattCoeffn(1:2,1)=coeffs(1:2); %An,Bn at  n=0 %SCALED coeffs(1)=An*hn(kc1rs)
        scattCoeffPn(1:2,1)=coeffs(7:8); %Gn,Hn at n=0 %SCALED coeffs(7)=Gn*hn(kc2*rc)
                            
        hnk1rs=spherFactor*(besselh(0.5,1,K1rs)/sqrt(K1rs)); %=hn(kc1*rs)
        hnk2rc=spherFactor*(besselh(0.5,1,K2rc)/sqrt(K2rc)); %=hn(kc2*rc)
    
        scattCoeffn(1,1)=coeffs(1)/hnk1rs; %An,Bn at  n=0 %UNSCALED coeffs(1)=An*hn(kc1rs)
        scattCoeffPn(1,1)=coeffs(7)/hnk2rc; %Gn,Hn at n=0 %UNSCALED coeffs(7)=Gn*hn(kc2*rc)

        
    else %n>0
        coeffs=zeros(1,12);
        coeffs=LHS\RHS;
 
        if ECAHOption
            %calculate just the ECAH solution with core material and zero
            %shell thickness
            coeffs([1 2 3 7 8 9])=LHS6_6\RHS6_6;
        end
  
        
        if noTherm %if requested, remove all thermal terms and bdry condns
            if noShear %no thermal and no shear 4x4
                LHSNew2=LHS([1 5 7 11],[1 4 7 10]);
                RHSNew2=RHS([1 5 7 11]);
                coeffsNew2=LHSNew2\RHSNew2;
                coeffs(1)=coeffsNew2(1);
                       
                                
            else %no thermal, but include shear 8x8
                LHSNew2=LHS([1 2 5 6 7 8 11 12],[1 3 4 6 7 9 10 12]);
                RHSNew2=RHS([1 2 5 6 7 8 11 12]);
                coeffsNew2=LHSNew2\RHSNew2;
                coeffs(1)=coeffsNew2(1);
                
            end
        else %include thermal
            if noShear %exclude shear waves
                LHSNew2=LHS([1 3 4 5 7 9 10 11],[1 2 4 5 7 8 10 11]);
                RHSNew2=RHS([1 3 4 5 7 9 10 11]);
                coeffsNew2=LHSNew2\RHSNew2;
               coeffs(1)=coeffsNew2(1);
            
            end
        end
        %}
        scattCoeffn(1:3,n+1)=coeffs(1:3);  %An,Bn,Cn for n>0 SCALED coeffs(1)=An*hn(kc1rs)
        scattCoeffPn(1:3,n+1)=coeffs(7:9); %Gn,Hn,In for n>0%SCALED coeffs(7)=Gn*hn(kc2*rc
%STORE the UNSCALED coefficients for compressional wavemodes for consistency 
%with other models and to make the multiple scattering calculation easier
        hnk1rs=spherFactor*(besselh(n+0.5,1,K1rs)/sqrt(K1rs)); %=hn(kc1*rs)
        hnk2rc=spherFactor*(besselh(n+0.5,1,K2rc)/sqrt(K2rc)); %=hn(kc2*rc)
        scattCoeffn(1,n+1)=coeffs(1)/hnk1rs; %An,Bn at  n=0 %UNSCALED coeffs(1)=An*hn(kc1rs)
        scattCoeffPn(1,n+1)=coeffs(7)/hnk2rc; %Gn,Hn at n=0 %UNSCALED coeffs(7)=Gn*hn(kc2*rc)

   
%         %%%%%%%%%%%%%%%%%%%%%%%%%To compare to ECAH6x6:%%%%%%%%%%%%%%%%
%         coeffs=zeros(1,6);
%         coeffs(1:6)=LHS6_6\RHS6_6;
%         coeffs(2)=coeffs(2)*(1/L1b^3);
%         coeffs(3)=coeffs(3)*(1/S1b);
%         coeffs(5)=coeffs(5)*(1/L1a^3);
%         coeffs(6)=coeffs(6)*(1/S3a);
%         
%         scattCoeffn(1:3,n+1)=coeffs(1:3);  %An,Bn,Cn for n>0
%         scattCoeffPn(1:3,n+1)=coeffs(4:6); %Gn,Hn,In for n>0
%  %%%%%%%%%%%%%%%%%%%%%%%%%To compare to ECAH6x6:%%%%%%%%%%%%%%%%
    end %end of if n==0 block
    
end %end of for n=0:nLast block
end

