function [scattCoeffn,scattCoeffPn,matrixNearSing,nMax_OK]=...
    scattCoeffECAH...
    (nMax,propsCont,propsDisp,f,a,jn0,hn,jn,K,L,S,Kp,Lp,Sp,...
    gammaC,gammaT,gammaCp,gammaTp,tempK,thermOption)

% ECAH solutions by Valerie Pinfield 2004-2024 for single dispersed phase substance,
% and a single frequency and size
% calculate the scattering coefficients An etc 
% adapted with multi-mode scattering coefficients 
% for thermal-compressional scattering e.g. T0CT,T0TC,T0TT and
% for shear-compressional scattering e.g. T1SC, T1CS, T1SS
% ECAH: Epstein Carhart 1953, Allegra Hawley 1972
% Multi mode coefficients developed by Valerie Pinfield and updated 2024
%
%Inputs:
%nMax:maximum order n, calculates to nMax+1 and array index is (n+1) 
%propsCont and propsDisp are structures for properties of continuous and
%dispersed phases,with (at least) fields 'density',
%'shearViscMod','conduct','matType'
%f:freq in MHz, a: radius in metres,
%jn0(iDeriv+1,n+1): incident field jn(Ka) for derivative iDeriv and order n
%hn(iMode,iDeriv+1,n+1) for cont phase field modes (spherical Hankel)
%jn(iMode,iDeriv+1,n+1) for disp phase field modes (spherical Bessel)
%K,L,S: wavenumbers of propl, thermal, shear modes in cont phase
%Kp,Lp,Sp: wavenumbers of propl, thermal, shear modes in disp phase
%gammaC,T,S: thermal factors for propl, thermal, shear modes, contin phase
%gammaCp,Tp,Sp: similarly for disp phase
% thermOption true/false to include/exclude thermal terms in boundary
% equations
%
%Outputs:
% Modes are 1-3 for propl (compressional), thermal and shear modes (modes 1-3)
%scattCoeffn(mode 1:3,n+1)=coefficients for partial wave order n for 
% incident compressional wave
% = An,Bn,Cn (ECAH) or equivalently TnCC,TnCT, TnCS
% in T-matrix notation (see e.g. Luppe, Conoir, Valier-Brasier 2022)
% scattCoeffPn(mode 1:3,n+1)=coefficients for partial wave order n for 
% incident thermal (n=0) or shear wave (n>=1)
% matrixNearSing is a flag='true' if any matrix was nearly singular

warning('off','MATLAB:nearlySingularMatrix'); %turn off warnings about near-singular matrices
matrixNearSing=false; %set false flag for this call to function
nMax_OK=true;

scattCoeffn=zeros(3,nMax+2);
scattCoeffPn=zeros(3,nMax+2);

iOmega=1i*2e6*pi*f; %f is in MHz
pressFac=iOmega*propsCont.density;
%equations are written as for liquids
%for solids, cconvert viscosity and multiply by i*omega
%dispersed phase type
if strcmpi(propsDisp.matType,'solid')
    isCavity=0;
    etaP=propsDisp.shearViscMod/(-iOmega);
elseif strcmpi(propsDisp.matType,'cavity')
    isCavity=1;
else %liquid
    isCavity=0;
    etaP=propsDisp.shearViscMod;
end
%continuous phase type
if strcmpi(propsCont.matType,'solid')
    etaC=propsCont.shearViscMod/(-iOmega);
else %liquid
    etaC=propsCont.shearViscMod;
end

if ~isCavity
    etaHat=etaP/etaC;
    torHat=propsDisp.conduct/propsCont.conduct;
    rhoHat=propsDisp.density/propsCont.density;
end;
% Bulk viscosity is included for fluids through the attenuation
% For solids, corresponds to a complex bulk modulus

Ka=K*a;
La=L*a;
Sa=S*a;
Kpa=Kp*a;
Lpa=Lp*a;
Spa=Sp*a;
LHS=zeros(6,6);
RHS=zeros(6,1);
coeffs=zeros(1,6);

%Check the number of orders needed
if(abs(Ka)>0.5)
    nReqd=ceil(abs(Ka))+5; %Ka+5 is the number of orders needed
elseif (abs(Ka)<0.05)
    nReqd=2; %only orders up to 2 if in long wavelength limit. Solids need n=2 too
    nReqd=nMax+1; %just calculate to nMax+1
else
    nReqd=nMax; %otherwise do as requested
end

if (nReqd>(nMax+1))
    nLast=nMax+1;  %only up to full array size
    if ~alreadyAskednMax %if haven't warned the user yet, ask if continue or abort
        alreadyAskednMax=true; %flag up that the user has been warned
        [OK_Cancel]=questdlg(sprintf('%40s\n%40s','nMax is not big enough. Need nMax>Ka+5:',...
            'Click OK to continue or Cancel to abort calculation'),...
            'Inaccurate calculation','OK','Cancel','OK');
        if strcmp(OK_Cancel,'Cancel')
            nMax_OK=false;
            return
        end
    end
else
    nLast=nReqd; %usual calculation
end

%*************************************************************************
%split solution into cavities or not
if isCavity %the dispersed phase is a cavity
    % no wave modes in cavity.
    %displacement boundary conditions invalid(displacement not constrained)
    %stress components are zero at boundary
    rhoHat=1.0;
    %***************** Cycle through orders n
    for n=0:nLast %each order
        nn_1=n*(n+1);
        n2n=n*n+n-2;
        LHS=zeros(6,6);
        RHS=zeros(6,1);
        coeffs=zeros(1,6);
        %UNCONDITIONED ECAH MATRIX modified for cavity
        %%{
        %do a non-thermal solution, just propagational and shear mode - 2 equations
        %A,C, Prr, PrTheta
        %equation 1 is radial stress Prr: zero at boundary (eqn 4 in full soln)
        LHS(1,1)=(hn(1,1,n+1)-2.0*etaC*K*K*(hn(1,3,n+1)+hn(1,1,n+1))/pressFac);
        LHS(1,2)=(2.0*etaC*nn_1*(Sa*hn(3,2,n+1)-hn(3,1,n+1))/(pressFac*a*a));
        %equation 2 is radial stress Prtheta: zero at bdry (eqn 6 in full soln)
        LHS(2,1)=Ka*hn(1,2,n+1)-hn(1,1,n+1);
        LHS(2,2)=(-0.5*(Sa*Sa*hn(3,3,n+1)+n2n*hn(3,1,n+1)));
        %right hand side are from incident field
        RHS(1,1)=(-jn0(1,n+1)+2.0*etaC*K*K*(jn0(3,n+1)+jn0(1,n+1))/pressFac);
        RHS(2,1)=jn0(1,n+1)-Ka*jn0(2,n+1);

        if n==0
            scattCoeffn(1,1)=RHS(1,1)/LHS(1,1);
        else
            if n==1
                %improve accuracy by using relation  for Bessel functions
                %xf'(n)-nf(n)=-xf(n+1)
                LHS(2,1)=-Ka*hn(1,1,3);
                RHS(2,1)=Ka*jn0(1,3);
                LHS(1,2)=(-2.0*etaC*nn_1*Sa*hn(3,1,3)/(pressFac*a*a));
            end

            LHS2_2=LHS(1:2,1:2);
            RHS2_2=RHS(1:2,1);

            coeffs(1:2)=LHS2_2\RHS2_2;
            if(rcond(LHS2_2)<eps) % set flag to show inaccuracy of inversion
                matrixNearSing='true';
            end
            scattCoeffn(1,n+1)=coeffs(1);
        end

        %}
        % End of non-thermal cavity solution

        
        %{
        % ***************** Optional solution for cavity with thermal terms
        %A (propagational mode), B (thermal),C (shear)
        %equation 1 is heat flux: zero at boundary (eqn 3 in full soln)
        LHS(1,1)=Ka*hn(1,2,n+1); %order n, hn' (first deriv);
        LHS(1,2)=gammaT*La*hn(2,2,n+1)/gammaC;
        LHS(1,3)=0.0;
        %equation 2 is radial stress Prr: zero at boundary (eqn 4 in full soln)
        LHS(2,1)=(hn(1,1,n+1)-2.0*etaC*K*K*(hn(1,3,n+1)+hn(1,1,n+1))/pressFac);
        LHS(2,2)=(hn(2,1,n+1)-2.0*etaC*L*L*(hn(2,3,n+1)+hn(2,1,n+1))/pressFac);
        LHS(2,3)=(2.0*etaC*nn_1*(Sa*hn(3,2,n+1)-hn(3,1,n+1))/(pressFac*a*a));
        %equation 3 is radial stress Prtheta: zero at bdry (eqn 6 in full soln)
        LHS(3,1)=Ka*hn(1,2,n+1)-hn(1,1,n+1);
        LHS(3,2)=La*hn(2,2,n+1)-hn(2,1,n+1);
        LHS(3,3)=(-0.5*(Sa*Sa*hn(3,3,n+1)+n2n*hn(3,1,n+1)));

        %right hand side are from incident field
        RHS(1,1)=-Ka*jn0(2,n+1);
        RHS(2,1)=(-jn0(1,n+1)+2.0*etaC*K*K*(jn0(3,n+1)+jn0(1,n+1))/pressFac);
        RHS(3,1)=jn0(1,n+1)-Ka*jn0(2,n+1);
    
        if n==0 %do the n=0 solution, no shear stress cpt, only 2 bdry eqns
    
            LHS2_2=LHS(1:2,1:2);
            RHS2_2=RHS(1:2,1);
    
            coeffs(1:2)=LHS2_2\RHS2_2;   %this is the solution ******* n=0
            if(rcond(LHS2_2)<eps) % set flag to show inaccuracy of inversion
                matrixNearSing='true';
            end
            %{
            %NO THERMAL TERMS
            coeffs(1)=-RHS(1,1)/LHS(2,1); 
            %}
            %*********************************
            % Solution with thermal terms for cavity n=0
            scattCoeffn(1:2,1)=coeffs(1:2); %modes 1&2 are An and Bn for n=0
            scattCoeffn(3,1)=0.0; %mode 3 (Cn) for n=0
            scattCoeffPn(1:3,1)=0.0;%all internal modes zero for cavity
 
        else % All higher orders (for cavity) n>0
            if n==1
            %improve accuracy by using relation  for Bessel functions
            %xf'(n)-nf(n)=-xf(n+1)
            LHS(3,1)=-Ka*hn(1,1,3);
            LHS(3,2)=-La*hn(2,1,3);
            RHS(3,1)=Ka*jn0(1,3);
            LHS(2,3)=(-2.0*etaC*nn_1*Sa*hn(3,1,3)/(pressFac*a*a));
        end
            LHS3_3=LHS(1:3,1:3);
            RHS3_3=RHS(1:3,1);
    
            coeffs(1:3)=LHS3_3\RHS3_3;   %this is the solution ******* n>0
            if(rcond(LHS3_3)<eps) % set flag to show inaccuracy of inversion
                matrixNearSing='true';
            end

        %******************
        % Full ECAH solution for cavity n>0 with thermal terms
        coeffs=LHS3_3\RHS3_3; %this is the full ECAH 3x3 solution for cavity
        
        scattCoeffn(1:3,n+1)=coeffs(1:3); %modes 1:3 are An,Bn,Cn for n
        scattCoeffPn(1:3,n+1)=0.0; % all modes zero inside cavity

        end % end of if n==0 block
        %}
        % End of cavity solution with thermal terms
        %*************************************
        
    end    % end of for n=0:nLast block for cavity
    
else % not a cavity
    %**********************************************************************
    % Cycle through orders n
    for n=0:nLast %each order
        nn_1=n*(n+1);
        n2n=n*n+n-2;
        LHS=zeros(6,6);
        RHS=zeros(6,1);
        coeffs=zeros(1,6);

        %UNCONDITIONED ECAH MATRIX 
        %matrix coefficients, order A, B,A', B',C,C' 
        % Non-primed is for external coefficients, 
        % Prime is for internal coefficients, 
        % order A-C is compresional, thermal, shear modes
        
        %equation 1 radial VELOCITY
        LHS(1,1)=Ka*hn(1,2,n+1); %order n, hn' (first deriv)
        LHS(1,2)=La*hn(2,2,n+1);
        LHS(1,5)=-nn_1*hn(3,1,n+1);
        LHS(1,3)=-Kpa*jn(1,2,n+1);
        LHS(1,4)=-Lpa*jn(2,2,n+1);
        LHS(1,6)=nn_1*jn(3,1,n+1);
        %equation 5 is tangential VELOCITY
        LHS(5,1)=hn(1,1,n+1);
        LHS(5,2)=hn(2,1,n+1);
        LHS(5,5)=-(hn(3,1,n+1)+Sa*hn(3,2,n+1));
        LHS(5,3)=-jn(1,1,n+1);
        LHS(5,4)=-jn(2,1,n+1);
        LHS(5,6)=(jn(3,1,n+1)+Spa*jn(3,2,n+1));
        %equation 2 is temperature
        LHS(2,1)=LHS(5,1);
        LHS(2,2)=gammaT*LHS(5,2)/gammaC;
        LHS(2,5)=0.0;
        LHS(2,3)=gammaCp*LHS(5,3)/gammaC;
        LHS(2,4)=gammaTp*LHS(5,4)/gammaC;
        %equation 3 is heat flux
        LHS(3,1)=LHS(1,1);
        LHS(3,2)=gammaT*LHS(1,2)/gammaC;
        LHS(3,5)=0.0;
        LHS(3,3)=torHat*gammaCp*LHS(1,3)/gammaC;
        LHS(3,4)=torHat*gammaTp*LHS(1,4)/gammaC;
        LHS(3,6)=0.0;
        %equation 4 is radial stress Prr
        LHS(4,1)=(hn(1,1,n+1)-2.0*etaC*K*K*(hn(1,3,n+1)+hn(1,1,n+1))/pressFac);
        LHS(4,2)=(hn(2,1,n+1)-2.0*etaC*L*L*(hn(2,3,n+1)+hn(2,1,n+1))/pressFac);
        LHS(4,3)=(-rhoHat*jn(1,1,n+1)+2.0*etaP*Kp*Kp*(jn(1,3,n+1)+...
            jn(1,1,n+1))/pressFac);
        LHS(4,4)=(-rhoHat*jn(2,1,n+1)+2.0*etaP*Lp*Lp*(jn(2,3,n+1)+jn(2,1,n+1))...
            /pressFac);
        LHS(4,5)=(2.0*etaC*nn_1*(Sa*hn(3,2,n+1)-hn(3,1,n+1))/(pressFac*a*a));
        LHS(4,6)=(-2.0*etaP*nn_1*(Spa*jn(3,2,n+1)-jn(3,1,n+1))/(pressFac*a*a));
        %equation 6 is radial stress Prtheta
        LHS(6,1)=LHS(1,1)-LHS(5,1);
        LHS(6,2)=LHS(1,2)-LHS(5,2);
        LHS(6,5)=(-0.5*(Sa*Sa*hn(3,3,n+1)+n2n*hn(3,1,n+1)));
        LHS(6,3)=etaHat*(LHS(1,3)-LHS(5,3));
        LHS(6,4)=etaHat*(LHS(1,4)-LHS(5,4));
        LHS(6,6)=(0.5*etaHat*(Spa*Spa*jn(3,3,n+1)+n2n*jn(3,1,n+1)));

        %right hand side are from incident field
        % Compressional incident wave
        RHS(1,1)=-Ka*jn0(2,n+1);
        RHS(5,1)=-jn0(1,n+1);
        RHS(2,1)=RHS(5,1);
        RHS(3,1)=RHS(1,1);
        RHS(4,1)=(-jn0(1,n+1)+2.0*etaC*K*K*(jn0(3,n+1)+jn0(1,n+1))/pressFac);
        RHS(6,1)=RHS(1,1)-RHS(5,1);

        % ****************************************************************
        if n==0 %do the n=0 solution, only 4 boundary equations, no shear

            LHS4_4=LHS(1:4,1:4);
            RHS4_4=RHS(1:4,1);

            coeffs(1:4)=LHS4_4\RHS4_4;   %this is the solution ******* n=0
            if(rcond(LHS4_4)<eps) % set flag to show inaccuracy of inversion
                matrixNearSing='true';
            end
            %******************************************
            scattCoeffn(1,1)=coeffs(1); %modes 1 is An for n=0 (T0CC)
            %Store the SCALED thermal coefficient = T0CT*h0(kTa)
            scattCoeffn(2,1)=coeffs(2)*hn(2,1,1); % mode 2 is Bn for n=0 (scaled)
            %scattCoeffn(2,1)=coeffs(2); %Option to store the UNSCALED coefficient
            scattCoeffn(3,1)=0.0; %mode 3 (Cn) for n=0
            % Store internal coeffs - later overwritten for mode-conversion
            % coefficients instead (incident waves of other modes)
            scattCoeffPn(1:2,1)=coeffs(3:4); %modes 1&2 An' and Bn' 
            scattCoeffPn(3,1)=0.0; % Cn' for n=0

            %******************************************
            % %{
            % Optional non-thermal solution which is good for solid
            % scatterers in liquids. For n=0
            % Used for PRSA2024 papers
            if ~thermOption %chosen non-thermal option
            LHS2_2=LHS([1 4],[1 3]);
            RHS2_2=RHS([1 4],1);
            coeffs2(1:2)=LHS2_2\RHS2_2;
            scattCoeffn(1,1)=coeffs2(1); %T0CC with no thermal contributions
            %*****************Non-thermal solution completed
            end
            %}

            %********************************************
            %{
            % Multi-mode scattering coefficients for n=0
            % Compressional-thermal
            % Compressional incident, thermal scattered 
            % Then thermal  incident, compressional scattered, 
            % and thermal scattered

            %*******Incident thermal wave, numerical matrix solution
            %calculate the Bessel functions for the incident thermal wave
            j0th=sin(La)/La;
            j0pth=(cos(La)-j0th)/La;
            j0dpth=-j0th-2*j0pth/La;

            %Now construct the right hand side for the thermal incident wave
            RHSth(1,1)=-La*j0pth; % eqn 1 is vr
            RHSth(2,1)=-gammaT*j0th/gammaC; %eqn 2 is temperature scaled by gammaC
            RHSth(3,1)=-gammaT*La*j0pth/gammaC; %eqn 3 is heat flux scaled by tau*gammaC
            RHSth(4,1)=-j0th+(2*etaC*L*L/pressFac)*(j0dpth+j0th); %eqn 4 is Prr scaled by eta*Sa*Sa

            %solve the matrix equation again for thermal incident wave
            coeffsth(1:4)=LHS4_4\RHSth;
            coeffsth(1); %the scattered compr amplit for thermal incident n=0

            % *************************
            %Store results for thermal incident wave
            %Calculated by numerical solution of the boundary equations
            %This now stores the scattering coefficients for the thermal incident wave
            scattCoeffPn(1,1)=coeffsth(1)/j0th; % 1 is the scattered compressional mode T0TC/j0(kTa)(scaled)
            %the outgoing scattered thermal wave coefficient is stored scaled as T0TT*h0(kTa)/j0(kTa) 
            scattCoeffPn(2,1)=coeffsth(2)*hn(2,1,1)/j0th; %is the scattered thermal mode (i.e.scaled)
            %scattCoeffPn(1:2,1)=coeffsth(1:2); %Option to store the UNSCALED coefficient

            % **************************
            % Optional analytical asymptotic solutions for n=0 
            % Comment out the %{ on the next line to avoid these
            %{
            % Analytical solution for T0CT
            % the thermal scattered wave coefficient
            % with compressional incident wave for small kca
            % the analytical result is for scaled thermal scattered waves by h0(kTa)=-i*exp(i*kTa)/kTa
            % =T0CT*h0(La)=T0CT*h0(kTa)
            b2=(gammaT/gammaC)*(1-La*hn(2,2,1)*jn(2,1,1)/...
                (torHat*Lpa*hn(2,1,1)*jn(2,2,1)));
            T0CTSc=-(1-gammaCp/(gammaC*rhoHat))/b2;

            % Stored coefficient options
            % scattCoeffn(2,1)=T0CTSc; % Scaled analytical result T0CT*h0(kTa)
            % scattCoeffn(2,1)=T0CTSc/hn(2,1,1); % Unscaled analytical result T0CT

            % Analytical asymptotic solution for thermal incident wave,
            
            % Calculate the relevant derivatives for the Bessel/Hankel
            % functions
            j0pTpS=jn(2,2,1)/jn(2,1,1);
            h0pTS=hn(2,2,1)/hn(2,1,1);

            %full analytical solution for T0TC 
            % (scaled by j0(La) i.e. T0TCscl=T0TC/j0(La)
            T0TCSc=1i*Ka*La*(j0pth/j0th)*(1-gammaT/(torHat*gammaTp))*...
                (1-h0pTS*j0th/j0pth)/(1-(La/(Lpa*torHat))*h0pTS/j0pTpS);
            %analytical solution for T0TT
            %scaled incident thermal wave, scaled scattered wave
            T0TTSc=-((j0pth/j0th)*La-Lpa*torHat*j0pTpS)/(La*h0pTS-Lpa*torHat*j0pTpS);
            T0TT=j0th*T0TTSc/hn(2,1,1);

            %   stored coefficient options for analytical multimode
            %   coefficients
            % scattCoeffPn(1,1)=T0TCSc; %full scaled analytical result
            % scattCoeffPn(1,1)=T0TCSc*j0th; %full scaled analytical result multiplied by j0(La) so unscaled T0TC

            %}
            % End of analytical solutions for multi-mode coefficients 
            % for n=0

            %}
            % End of optional multi-mode coefficient calculations for n=0
            
        % ******************************************************************
        else %**************** All higher orders n>0
            if n==1
                %improve accuracy by using relation  for Bessel functions
                %xf'(n)-nf(n)=-xf(n+1)
                LHS(6,1)=-Ka*hn(1,1,3);
                LHS(6,2)=-La*hn(2,1,3);
                LHS(6,3)=Kpa*jn(1,1,3)*etaHat;
                LHS(6,4)=Lpa*jn(2,1,3)*etaHat;
                RHS(6,1)=Ka*jn0(1,3);
                LHS(4,5)=(-2.0*etaC*nn_1*Sa*hn(3,1,3)/(pressFac*a*a));
                LHS(4,6)=(2.0*etaP*nn_1*Spa*jn(3,1,3)/(pressFac*a*a));
            end

            coeffs=LHS\RHS; %this is the full ECAH 6x6 solution ***********
            
            % *************************
            %Store coefficients for order n, scaled as appropriate
            scattCoeffn(1,n+1)=coeffs(1); % TnCC or An (mode 1) 
            %TnCSscaled=TnCS*hn(Sa)=TnCS*hn(kSa)
            scattCoeffn(3,n+1)=coeffs(5)*hn(3,1,n+1); %mode 3 (Cn) for n SCALED by hn(Sa)
            % Formerly, the internal coefficients were stored, as below
            % These are now superceded by the multi-mode coefficients
            % for incident thermal and shear waves
            scattCoeffPn(1,n+1)=coeffs(3); %mode 1 compressional internal coeff An' (later overwritten)
            %scattCoeffPn(2,n+1)=coeffs(4); %mode 2 thermal internal coeff Bn'(unused)
            scattCoeffPn(3,n+1)=coeffs(6); % mode 3 shear internal coeff Cn' (later overwritten)

            %******************************
            % Optional non-thermal solution for n>0. Numerical matrix soln
            % %{
            if ~thermOption %chosen non-thermal option
            %Bdry eqns are 1:ur; 2: temp 3: heat flux 4: Prr 5:utheta 6:prtheta
            %coeffs A B A' B' C C' -> A A' C C'
            LHS44=LHS([1 4 5 6],[1 3 5 6]);
            RHS44=RHS([1 4 5 6],1);
            coeffs4=LHS44\RHS44;

            % Store non-thermal numerical matrix soln, scaled
            scattCoeffn(1,n+1)=coeffs4(1); %TnCC
            %TnCSscaled=TnCS*hn(Sa)
            scattCoeffn(3,n+1)=coeffs4(3)*hn(3,1,n+1); % mode 3 (Cn) for n SCALED by hn(Sa)
            scattCoeffPn(1,n+1)=coeffs4(2); % An'
            scattCoeffPn(3,n+1)=coeffs4(4); % Cn' for n
            end
            %}
            %***************End of non-thermal solution for arbitrary n>=1         
            
            %*************************************************************************
            %For calculation of the multimode terms for n>=1 for incident shear
            %wave
            % March 2023
            %4x4 nonthermal version has coeff order A,A',C,C'
            % (comp external, comp internal, shear external, shear
            % internal)
            % (c.f.full set A,B,A',B',C,C')= columns of LHS
            % Eqn order: vr,vtheta,Prr, Prtheta
            % c.f. full eqn order vr, T,heat flux, Prr,vtheta, Prtheta
            % LHS([1 5 4 6],[1 3 5 6])

            jnSa=sqrt(pi/2)*besselj(n+0.5,Sa)/sqrt(Sa);
            hnSa=hn(3,1,n+1);
            jn_1Sa=sqrt(pi/2)*besselj(n-0.5,Sa)/sqrt(Sa); %to calc the derivs
            jnpSa=(jn_1Sa-(n+1)*jnSa/Sa); %jnp=jn_1-(n+1)*jn/x
            jndpSa=-(2/Sa)*jn_1Sa-jnSa+(n+1)*(n+2)*jnSa/(Sa*Sa);
            %RHS  by comparison with column 5 of LHS matrix but with jn not hn
            RHSsh(1,1)=nn_1*jnSa; %c.f. eqn 1 vr, col 5 (C')
            RHSsh(2,1)=(jnSa+Sa*jnpSa); %c.f. eqn 5 (vtheta) col 5 (C')
            RHSsh(3,1)=-(2.0*etaC*nn_1*(Sa*jnpSa-jnSa)/(pressFac*a*a)); %c.f. eqn 4 Prr, col 5 C'
            if n==1 RHSsh(3,1)=-(2.0*etaC*nn_1*(-sqrt(pi/2)*Sa*besselj(n+1.5,Sa)/sqrt(Sa))/(pressFac*a*a));end
            RHSsh(4,1)=0.5*(Sa*Sa*jndpSa+n2n*jnSa); %c.f.eq 6 Prtheta, col 5 C'

            %extract from LHS the right rows and columns to remove thermal
            LHSsh=LHS([1 5 4 6],[1 3 5 6]);
            coeffsSh=LHSsh\RHSsh; %this is the ECAH 4x4 solution inc shear wave no thermal ****************

            %Store SCALED coefficients for all the non-thermal results for
            %incident shear wave

            scattCoeffPn(1,n+1)=coeffsSh(1)/jnSa;%TnSC scaled=TnSC/jn(Sa)
            scattCoeffPn(3,n+1)=coeffsSh(3)*hnSa/jnSa; %TnSS
            %Mar 2023 These formulae are correct now, checked against analytical results for
            %small ka, any sa, and checked scaling with n for kca^(f(n)) for n>1.

            % *******************
            %{
            % Optional analytical coefficients for n=1 only 
            % for incident shear and incident compressional
            % with non-thermal boundary equations and small kca
            if n==1
                % Calculate the spherical Bessel functions for shear waves
                j0sh=sin(Sa)/Sa;
                j1sh=(1/Sa)*(sin(Sa)/Sa-cos(Sa));
                j2sh=(1/Sa)*((3/(Sa*Sa)-1)*sin(Sa)-(3/Sa)*cos(Sa));

                % Unscaled coefficients
                denom=3*hn(3,1,3)-2*(rhoHat-1)*hn(3,1,1);
                T1CC=1i*(Ka*Ka*Ka/3)*(rhoHat-1)*hn(3,1,3)/denom;
                T1CS=-(Ka/Sa)*(rhoHat-1)/denom;
                T1SC=-(2i*Sa*Ka*Ka/3)*(rhoHat-1)*(hn(3,1,3)*j0sh-...
                    hn(3,1,1)*j2sh)/denom;
                T1SS=-(3*j2sh-2*(rhoHat-1)*j0sh)/denom;

                %Save the analytical coefficients (scaled)
                scattCoeffn(1,n+1)=T1CC; % or An, n=1 inc comp, scattered comp
                scattCoeffn(3,n+1)=T1CS*hn(3,1,2); % mode 3 (Cn) for n SCALED by h1(Sa)
                scattCoeffPn(1,n+1)=T1SC/j1sh; %T1SC scaled by /j1(Ksa) inc shear scatt comp
                scattCoeffPn(3,n+1)=T1SS*hn(3,1,2)/j1sh; %T1SS *h1(ksa)/j1(ksa) inc shear scatt shear
            end
            %}
            % End of optional analytical coefficient for n=1
            %************************************************

        end % end of if n==0 block
    end    % end of for n=0:nLast block
end % end of if cavity block 

end %end of function

