function [KEff,vel,att,TnCoeff,K,gammaCT]=...
    scatterCalc3D(nMax,coeffOption,thermOption,multiMode,multiModeOptions,tempK,...
    physProps,freq,medRad,sigmaR,numBins,distType,volFractTot,exclFac)
%
% Execute scattering calculations for a 3D system
% with properties and parameters passed in as arguments
% the system can be set up using the script file
% There are 4 options for the scattering coefficient calculations
% and two different formulations of the multiple scattering calculations
% ******************************************
% Inputs:
% nMax: max partial wave order
% coeffOption: selected option for the scattering coefficient calculations
% coeffOption: 1=ECAH numerical solution of boundary equations matrices with
% options to turn on/off thermal effects and to select the analytical
% asymptotic expresions as an alternative. 
% ECAH=Epstein Carhart 1953, Allegra Hawley 1972
% coeffOption: 2=Anson-Chivers 1993 core-shell model operated as ECAH
% spherical scatterer
% coeffOption: 3=acoustic only scattering coefficient with no mode conversion
% coeffOption: 4=asymptotic analytics with mode-conversion spherical scatterer
% thermOption: 1/0 to include/exclude thermal effects in boundary
% conditions when calculating scattering coefficients
% multiMode: to switch on/off the multi-mode contributions to multiple scattering
% multiMode=false uses the single-mode multiple scattering formulation based
% on Lloyd/Berry 1967, Linton/Martin 2006, Gower, Smith, Parnell, Abrahams
% 2018 for multiple scatterer species
% multiMode=true uses the multi-mode multiple scattering formulation based
% on Luppe Conoir Valier Brasier 2022 for single species (single particle
% size)
% multiModeOptions=[0/1,0/1,0/1] Booleans to switch on/off terms
% thermal multimode scattering (on/off), shear multimode scattering (on/off)
% full dispersion relation(1) or low concentration expansion(0)
% tempK: equil. temperature/K
% physProps: a structure array physProps{numSubs} with fields
% matType='liquid'/'solid/'cavity';soundVel,density,shearViscMod (viscosity
% in Pa s or shear modulus in Pa); conduct J/m s K (thermal conductivity)
% heatCapCp,expans;alphaFac,alphaExp
% freq(numFreq*1) frequency in MHz,
% medRad(numMedRad*1),sigmaR(sumSigmaR*1). 
% Median radius and standard deviation of log-normal size distributions.
% All radius inputs in micrometres
% numBins: number of size bins in each size distribution
% distType='lognormal': type of size distribution model
% The system is analysed as a set of configurations taking the i'th size
% distribution from each substance, and for each distribution taking each
% volume fraction (the j'th for each substance).
% each configuration and volume fraction is calculated for all frequencies
% volFractTot(numSubs,numVol): the total volume fraction of each dispersed phase substance
% exclFac: the exclusion factor. Radius of excluded volume =
% 2*exclFac*radius
% ***********************************
% Outputs:
% KEff(numDist,numVol,numFreq,numModels),vel/att(numDist,numVol,numFreq,numModels)
% effective wavenumber, effective velocity, effective attenuation for 
% each size, volume fraction, frequency and multiple scattering model
% wavenumbers in "per metre", speeds in m/s, attenuation in Np/m
% TnCoeff(numSubs,numModes,numFreq,numDist,numBins,nMax+1): scattering coefficients
% K(numSubs,numModes,numFreq): wavenumbers in all media. 
% Modes: 1-3 =compressional, thermal, shear
% gammaCT(numSubs,numModes,numFreq): the thermal factors for all media

numSubs=size(medRad,1); %number of substances including continuous phase
numMedR=size(medRad,2); %number of different median radii
numSigma=size(sigmaR,2);%number of different distribution widths
numDist=numMedR*numSigma;
numModes=3;
numFreq=length(freq);
numVol=size(volFractTot,2); %volFractTot is (numSubs,numVol); numVol is number of volume fractions of each substance

%radiusType(numSubs,numDist,numBins) - the radii of each size or size bins
%for each substance in microns
%fractVol(numSubs,numDist,numBins) fractional volumes (sum to unity for
%each distribution)
[radiusType,fractVol] = setUpSysParams(medRad,sigmaR,numBins,distType);
%now convert to metres
radiusType=radiusType*1E-6; %in metres
%***RADII IN METRES from now on, FREQ IN MHz

%WAVEnumbers for each substance, mode, frequency: K(substance,mode,freq)
K=waveNumbers(tempK,physProps,freq); %wavenumbers in m^-1, freq in MHz
%factors for the thermal waves for each substance,mode, frequency gammaCT(substance,mode,freq)
gammaCT=thermalFactors(tempK,physProps,K,freq); %freq in MHz

%needs all radii in a single row or column for each substance and loop over
%substances; permute to cycle through bins first then dist
%Radii are in metres here
numSizes=numDist*numBins;
allRad=reshape(permute(radiusType,[1 3 2]),numSubs,numBins*numDist); %numsSubs*numSizes where numSizes=numDist*numBins
numOptions=1;
TnCoeff=zeros(numOptions,numSubs,numModes,numFreq,numSizes,numVol,nMax+2,2);%last element redundant for spheres
TnpCoeff=zeros(numOptions,numSubs,numModes,numFreq,numSizes,numVol,nMax+2,2);

for iSubs=2:numSubs %do each substance (iSubs=1 is continuous phase)
    %*********************************************************************
    %switch between different scattering coefficient calculations
    switch coeffOption
        case 1 %coeffOption=1: full 3-mode solution using ECAH coefficient

            % Call the Bessel function calculation for each substance
            % jn0(iDeriv,n,iFreq,iRad)
            % jnhn(iSubs,iMode,iDeriv,n,iFreq,iRad)
            KiSubs=zeros(3,size(K,2),size(K,3));
            KiSubs(1,:,:)=K(1,:,:); %use subs 1 as continuous phase
            KiSubs(2,:,:)=K(iSubs,:,:); %use "iSubs" as subs 2 the shell
            KiSubs(3,:,:)=K(iSubs,:,:); %also use iSubs as subs 3 the core

            jn0=zeros(3,nMax+2,numFreq,numSizes);
            [jn0(:,:,:,:),jnhn(:,:,:,:,:,:)]=...
                Besseljnhn(nMax+1,KiSubs,freq,squeeze(allRad(iSubs,:))); 
            
            for iFreq=1:numFreq %each frequency value
                for iDist=1:numDist %each distribution
                    for iBin=1:numBins %loop over size bins
                        iSizes=(iDist-1)*numBins+iBin; %array element in radius array
                        for iVol=1:numVol %each volume fraction
                            %use the TnCoeff and TnCoeffP arrays to store scatt coeffs
                            %nearSing is a flag true/false to warn of near-singular matrices

                            % Calculate the scattering coefficients using the
                            % standard ECAH model or using the asymptotic multimode
                            % coefficients (available by editing the
                            % function scattCoeffECAH)
                            [tempCoeff,temppCoeff,nearSing,nMax_OK]=...
                                feval('scattCoeffECAH',nMax,physProps{1},...
                                physProps{iSubs},freq(iFreq),...
                                radiusType(iSubs,iDist, iBin),...
                                squeeze(jn0(:,:,iFreq,iSizes)),...
                                squeeze(jnhn(1,:,:,:,iFreq,iSizes)),...
                                squeeze(jnhn(iSubs,:,:,:,iFreq,iSizes)),...
                                K(1,1,iFreq),K(1,2,iFreq),K(1,3,iFreq),...
                                K(iSubs,1,iFreq),K(iSubs,2,iFreq),K(iSubs,3,iFreq),...
                                gammaCT(1,1,iFreq),gammaCT(1,2,iFreq),...
                                gammaCT(iSubs,1,iFreq),gammaCT(iSubs,2,iFreq),tempK,thermOption);

                            %the code produces scattering coefficients up to
                            %nMax+1 so keep the 0:nMax+1 coefficients
                            TnCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=tempCoeff(:,1:nMax+2);
                            TnpCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=temppCoeff(:,1:nMax+2);

                        end %end of iVol=1:numVol loop (all volume fractions)
                    end %end of iBin loop (all particle size bins in distribution)
                end %end of iDist loop (all particle size distributions)
            end %end of iFreq=1:numFreq loop (all frequencies)

%*************************************************************************
        case 2 % coeffOption==2 
            %full 3-mode solution using Anson/Chivers core-shell as ECAH coeff
            %This uses the Anson-Chivers model but with matching core and shell - this
            %is my most stable code for calculating ECAH coefficients in
            %the 3-mode media
            
            thShell=0;

            %For each substance, calculate Bessel functions for a set of continuous phase, substance, and a core substance
            %if doing ECAH, the core duplicates the substance
            %create an array for subs1=subs1, subs2=iSubs, subs3=iSubs
            KiSubs=zeros(3,size(K,2),size(K,3));
            KiSubs(1,:,:)=K(1,:,:); %use subs 1 as continuous phase
            KiSubs(2,:,:)=K(iSubs,:,:); %use "iSubs" as subs 2 the shell
            KiSubs(3,:,:)=K(iSubs,:,:); %also use iSubs as subs 3 the core
            propsShell=physProps{iSubs}; %use iSubs as the shell properties
            propsCore=physProps{iSubs}; %use the iSubs as the core properties
            gammaCTiSubs=zeros(3,size(gammaCT,2),size(gammaCT,3));
            gammaCTiSubs(1,:,:)=gammaCT(1,:,:); %use subs 1 as continuous phase
            gammaCTiSubs(2,:,:)=gammaCT(iSubs,:,:); %use "iSubs" as subs 2 the shell
            gammaCTiSubs(3,:,:)=gammaCT(iSubs,:,:); %also use iSubs as subs 3 the core

            [jn1rs,hn1rs,jn2rs,jn2rc_rs,jnp2rc_rs,hn2rc,hn2rs_rc,hnp2rs_rc,jn3rc,limImkhTF]=...
                besselcoreshell(nMax+1,KiSubs,freq,...
                squeeze(allRad(iSubs,:)),thShell); %"allRad" is taken as the core radius, in metres
            shellRadius=allRad+thShell;
            for iFreq=1:numFreq %each frequency value
                for iDist=1:numDist %each distribution
                    for iBin=1:numBins %loop over size bins
                        iSizes=(iDist-1)*numBins+iBin; %array element in radius array
                        for iVol=1:numVol %each volume fraction
                            %use the TnCoeff and TnCoeffP arrays to store scatt coeffs
                            %nearSing is a flag true/false to warn of near-singular matrices
  
                            [tempCoeff,temppCoeff,nearSing,nMax_OK]=...
                                feval('scattCoeffACShell',nMax,physProps{1},...
                                propsShell,propsCore,freq(iFreq),...
                                squeeze(allRad(iSubs,iSizes)),squeeze(shellRadius(iSubs,iSizes)),...
                                squeeze(jn1rs(:,iFreq,iSizes)),...
                                squeeze(hn1rs(:,:,iFreq,iSizes)),...
                                squeeze(jn2rs(:,:,iFreq,iSizes)),...
                                squeeze(jn2rc_rs(:,:,iFreq,iSizes)),...
                                squeeze(jnp2rc_rs(:,:,iFreq,iSizes)),...
                                squeeze(hn2rc(:,:,iFreq,iSizes)),...
                                squeeze(hn2rs_rc(:,:,iFreq,iSizes)),...
                                squeeze(hnp2rs_rc(:,:,iFreq,iSizes)),...
                                squeeze(jn3rc(:,:,iFreq,iSizes)),...
                                squeeze(limImkhTF(:,iFreq,iSizes)),...
                                KiSubs(1,1,iFreq),KiSubs(1,2,iFreq),KiSubs(1,3,iFreq),...
                                KiSubs(2,1,iFreq),KiSubs(2,2,iFreq),KiSubs(2,3,iFreq),...
                                KiSubs(3,1,iFreq),KiSubs(3,2,iFreq),KiSubs(3,3,iFreq),...
                                gammaCTiSubs(1,1,iFreq),gammaCTiSubs(1,2,iFreq),...
                                gammaCTiSubs(2,1,iFreq),gammaCTiSubs(2,2,iFreq),...
                                gammaCTiSubs(3,1,iFreq),gammaCTiSubs(3,2,iFreq),1);
                            %last input is the "ECAHOption" to use AC model ECAH calcn

                            %the AC code produces scattering coefficients up to
                            %nMax+1 so just keep the 0:nMax+1 coefficients
                            TnCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=tempCoeff(:,1:nMax+2);
                            TnpCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=temppCoeff(:,1:nMax+2);

                        end %end of iVol=1:numVol loop (all volume fractions)
                    end %end of iBin loop (all particle size bins in distribution)
                end %end of iDist loop (all particle size distributions)
            end %end of iFreq=1:numFreq loop (all frequencies)

%**************************************************************************
        case 3 %coeffOption=3, 
            % i.e. use acoustic-only scattering coefficient (no thermal or
            % shear mode conversion)
            cSpd=physProps{1}.soundVel;
            cpSpd=physProps{iSubs}.soundVel;
            rho=physProps{1}.density;
            rhop=physProps{iSubs}.density;
            for iFreq=1:numFreq %each frequency value
                for iDist=1:numDist %each distribution
                    for iBin=1:numBins %loop over size bins
                        iSizes=(iDist-1)*numBins+iBin; %array element in radius array
                        for iVol=1:numVol %each volume fraction
                            %K(substance,mode,freq)
                            [tempCoeff]=feval('sphAcScattCoeff',nMax,...
                                K(1,1,iFreq),rho,K(iSubs,1,iFreq),rhop,cSpd,...
                                cpSpd,squeeze(allRad(iSubs,iSizes)));
                            %output tempCoeff is numFreq(1)*numRad(1)
                            %use the TnCoeff and TnCoeffP arrays to store scatt coeffs
                            %[TnCoeff(iOption,iSubs,:,iFreq,iSizes,iVol,:,1),...
                            TnCoeff(coeffOption,iSubs,1,iFreq,iSizes,iVol,:,1)=tempCoeff(:,:,1:nMax+2);
                            %TnpCoeff is redundant in this option but
                            %retain it for later processing consistency
                            TnpCoeff(coeffOption,iSubs,1,iFreq,iSizes,iVol,:,1)=tempCoeff(:,1:nMax+2);

                        end %end of iVol=1:numVol loop (all volume fractions)
                    end %end of iBin loop (all particle size bins in distribution)
                end %end of iDist loop (all particle size distributions)
            end %end of iFreq=1:numFreq loop (all frequencies)

%**************************************************************************
        case 4 % coeffOption==4 
            %analytical asymptotic scattering coefficients
            %thermal effects included or not, depending on thermOption
            %(1=include)

            %For each substance, calculate Bessel functions for a set of continuous
            %phase and substance 
            % Call the Besself function calculation for each substance
            %This is because the continuous phase functions are needed at the particle
            %sizes appropriate for each particle type (substance)
            %jn0(iDeriv,n,iFreq,iRad)
            %jnhn(iSubs,iMode,iDeriv,n,iFreq,iRad)
            KiSubs=zeros(3,size(K,2),size(K,3));
            KiSubs(1,:,:)=K(1,:,:); %use subs 1 as continuous phase
            KiSubs(2,:,:)=K(iSubs,:,:); %use "iSubs" as subs 2 the shell
            KiSubs(3,:,:)=K(iSubs,:,:); %also use iSubs as subs 3 the core

            jn0=zeros(3,nMax+2,numFreq,numSizes);
            [jn0(:,:,:,:),jnhn(:,:,:,:,:,:)]=...
                Besseljnhn(nMax+1,KiSubs,freq,squeeze(allRad(iSubs,:)));
            
            for iFreq=1:numFreq %each frequency value
                for iDist=1:numDist %each distribution
                    for iBin=1:numBins %loop over size bins
                        iSizes=(iDist-1)*numBins+iBin; %array element in radius array
                        for iVol=1:numVol %each volume fraction
                            %nearSing is a flag true/false to warn of near-singular matrices
                            
                            % Calculate the scattering coefficients using the
                            % standard ECAH model or using the asymptotic multimode
                            % coefficients
                            [tempCoeff,temppCoeff,nearSing,nMax_OK]=...
                                feval('scattCoeffAsympt',nMax,physProps{1},...
                                physProps{iSubs},freq(iFreq),...
                                radiusType(iSubs,iDist, iBin),...
                                squeeze(jn0(:,:,iFreq,iSizes)),...
                                squeeze(jnhn(1,:,:,:,iFreq,iSizes)),...
                                squeeze(jnhn(iSubs,:,:,:,iFreq,iSizes)),...
                                K(1,1,iFreq),K(1,2,iFreq),K(1,3,iFreq),...
                                K(iSubs,1,iFreq),K(iSubs,2,iFreq),K(iSubs,3,iFreq),...
                                gammaCT(1,1,iFreq),gammaCT(1,2,iFreq),...
                                gammaCT(iSubs,1,iFreq),gammaCT(iSubs,2,iFreq),tempK, thermOption);


                            %use the TnCoeff and TnCoeffP arrays to store scatt coeffs
                            %the code produces scattering coefficients up to
                            %nMax+1 so just keep the 0:nMax+1 coefficients
                            %[TnCoeff(iOption,iSubs,iMode,iFreq,iSizes,iVol,iOrder,1),...
                            %    TnpCoeff(iOption,iSubs,iMode,iFreq,iSizes,iVol,iOrder,1),nearSing,nMax_OK]
                            TnCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=tempCoeff(:,1:nMax+2);
                            TnpCoeff(coeffOption,iSubs,:,iFreq,iSizes,iVol,:,1)=temppCoeff(:,1:nMax+2);

                        end %end of iVol=1:numVol loop (all volume fractions)
                    end %end of iBin loop (all particle size bins in distribution)
                end %end of iDist loop (all particle size distributions)
            end %end of iFreq=1:numFreq loop (all frequencies)

    end %end of options for scattering coefficients
%**************************************************************************

end %end of iSubs=2:numSubs loop (all dispersed phase substance)

%END of calculations of scattering coefficients ***************************

%**************************************************************************
% Wavenumber calculations using various multiple scattering models
% Outputs: KEff(numDist,numVol,numFreq,numModels),
% velocity/atten(numDist,numVol,numFreq,numModels)

% TnCoeff and TnpCoeff are shape:
% numOptions,numSubs,numModes,numFreq,numSizes,numVol,(numOrders)calcnMax+1,2)

% pick one volume fraction, one calculation (i.e. final index=1)
T=squeeze(TnCoeff(coeffOption,:,:,:,:,1,:,1));%all sizes, all orders
%this is now numSubs,numModes,numFreq,numSizes,nMax+1
%reshape to split into size distributions
TT=permute(reshape(T,numSubs,numModes,numFreq,numBins,numDist,nMax+2),...
    [1 2 3 5 4 6]); %swap order round to get distribution and size bins in order (single option selected)
%TT is numSubs,numModes,numFreq,numDist,numBins,nMax+2

%For monodisperse systems - used for multi-mode scattering
TTMono=reshape(TT,numSubs,numModes,numFreq,numDist,[]); %remove the numBins dimension

%Similarly for the prime coefficients
TP=squeeze(TnpCoeff(coeffOption,:,:,:,:,1,:,1));%all sizes, all orders
%this is now numSubs,numModes,numFreq,numSizes,nMax+2
%reshape to split into size distributions
TTP=permute(reshape(TP,numSubs,numModes,numFreq,numBins,numDist,nMax+2),...
    [1 2 3 5 4 6]); %swap order round to get distribution and size bins in order (single option selected)
%TT is numSubs,numModes,numFreq,numDist,numBins,nMax+2
%For monodisperse systems - used for multi-mode scattering
TTPMono=reshape(TTP,numSubs,numModes,numFreq,numDist,[]); %remove the numBins dimension

% Multiple scattering models are
% 1st order in concn: Foldy (1)
% 2nd order in concn:
%    infinitely small scatterers (kb->0):
%           (2) no mode conversion mult scatt (LB67, LM06, GSPA18)
%           (3)	mode conversion mult scatt low concn expn (LCVB22)
%           (4)	mode conversion mult scatt dispersion relation (LCVB22)
%           (5)
%  	finite size scatterers
%           (6) no mode conversion mult scatt (GSPA18)
%           (7) mode conversion mult scatt low concn expn
%           (8) mode conversion mult scatt dispersion relation
%
numModels=8;

if ~multiMode
    %******Single mode multiple scattering, with/without volume exclusion
    % effects (without= point scatterers), with/without multiple species (Gower
    % et al. 2018), monodisperse or polydisperse
    % all volume fractions:
    [KEff, vel,att]=multScattmultSpecies(nMax,radiusType,fractVol,...
        volFractTot,freq,squeeze(K(1,1,:)),TT,exclFac,numModels);
    %KEff(numDist,numVol,numFreq,numModels),vel/att(numDist,numVol,numFreq,numModels)
else %multiMode==true
    %******Multiple mode multiple scattering
    %Luppe Conoir Valier Brasier 2022 + Pinfield/Valier-Brasier 2024
    %with/without volume exclusion terms
    % But only for monodisperse systems 
    % (there is no multi-species multi-mode model yet)
    if numBins==1
        %radiusType is in metres
        % radiusType(numSubs,numDist,numBins)
        if numSubs~=2
            warndlg("Multimode multiple scattering calculations can only be completed for a single dispersed phase substance")
        end
        [KEff,vel,att]=multScattSolnMultiMode(nMax,squeeze(radiusType),volFractTot,...
            freq,squeeze(K(1,1,:)),squeeze(K(1,2,:)),squeeze(K(1,3,:)),...
            TTMono,TTPMono,exclFac,numModels,multiModeOptions);
        %KEff(numDist,numVol,numFreq,numModels),vel/att(numDist,numVol,numFreq,numModels)
    else
        warndlg("Multimode multiple scattering calculations can only be completed for monodisperse systems (single particle size")
    end
end %end of multiple scattering options for multimode or not

end

