function [K]=waveNumbers(tempK,physProps,freq)
%calculates the wavenumbers for compressional, thermal and shear modes
%given the temperature, physical properties and frequency
%
%inputs are temperature, structure array physProps and a row vector of frequency values
% physProps has the following fields:
%matType, subsName,soundVel,density,shearViscMod,compressMod,conduct,...
%heatCapCp,expans
%
%output is an array of wavenumbers for compressional, thermal and shear modes
%in each substance included in the properties data
%K(substance,mode,freq)

omega=freq*1.0e6*2*pi; %angular frequency(row vector)/s (freq in MHz)
numSubs=size(physProps,2); %second dimension is number of substances
for iSubs=1:numSubs
    if(strcmpi(physProps{iSubs}.matType,'cavity'))
        K(iSubs,:,:)=0; %wavenumbers invalid for cavity
    else
        %thermal diffusivity
        diffus=physProps{iSubs}.conduct/...
            (physProps{iSubs}.density*physProps{iSubs}.heatCapCp);
        %ratio of heat capacities (gamma) -1/velocity^2
        soundVelocity=physProps{iSubs}.soundVel;
        gamma_1divv2=tempK*physProps{iSubs}.expans*physProps{iSubs}.expans/...
            physProps{iSubs}.heatCapCp;
        gamma=gamma_1divv2*soundVelocity*soundVelocity+1;
        K(iSubs,1,:)=((omega/soundVelocity))+(i*physProps{iSubs}.alphaFac*...
            freq(:)'.^(physProps{iSubs}.alphaExp)); %prop wavenumber
        K(iSubs,2,:)=sqrt(omega/(2*diffus))*(1+i); %thermal wavenumber
    
        if strcmpi(physProps{iSubs}.matType,'solid') %shear wavenumber, solid, shearViscMod is shear modulus
            K(iSubs,3,:)=omega*sqrt(physProps{iSubs}.density/(physProps{iSubs}.shearViscMod)); 
        else %shear wavenumber, liquid, shearViscMod is the viscosity
            K(iSubs,3,:)=sqrt(omega*physProps{iSubs}.density/(2*physProps{iSubs}.shearViscMod))...
                *(1+i); %shear wavenumber
        end
    end
end
